@@ -13,7 +13,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #if HAVE_CONFIG_H
@@ -52,14 +52,14 @@ static short BuffyNotify = 0;	/* # of unnotified new boxes */
 
 int fseek_last_message (FILE * f)
 {
-  long int pos;
+  LOFF_T pos;
   char buffer[BUFSIZ + 9];	/* 7 for "\n\nFrom " */
   int bytes_read;
   int i;			/* Index into `buffer' for scanning.  */
 
   memset (buffer, 0, sizeof(buffer));
   fseek (f, 0, SEEK_END);
-  pos = ftell (f);
+  pos = ftello (f);
 
   /* Set `bytes_read' to the size of the last, probably partial, buffer; 0 <
    * `bytes_read' <= `BUFSIZ'.  */
@@ -72,14 +72,14 @@ int fseek_last_message (FILE * f)
   {
     /* we save in the buffer at the end the first 7 chars from the last read */
     strncpy (buffer + BUFSIZ, buffer, 5+2); /* 2 == 2 * mutt_strlen(CRLF) */
-    fseek (f, pos, SEEK_SET);
+    fseeko (f, pos, SEEK_SET);
     bytes_read = fread (buffer, sizeof (char), bytes_read, f);
     if (bytes_read == -1)
       return -1;
     for (i = bytes_read; --i >= 0;)
       if (!mutt_strncmp (buffer + i, "\n\nFrom ", mutt_strlen ("\n\nFrom ")))
       {				/* found it - go to the beginning of the From */
-	fseek (f, pos + i + 2, SEEK_SET);
+	fseeko (f, pos + i + 2, SEEK_SET);
 	return 0;
       }
     bytes_read = BUFSIZ;
@@ -188,7 +188,7 @@ int mutt_parse_mailboxes (BUFFER *path, BUFFER *s, unsigned long data, BUFFER *e
       {
         FREE (&((*tmp)->path));
         tmp1=(*tmp)->next;
-        FREE (tmp);
+        FREE (tmp);		/* __FREE_CHECKED__ */
         *tmp=tmp1;
       }
       return 0;
@@ -212,7 +212,7 @@ int mutt_parse_mailboxes (BUFFER *path, BUFFER *s, unsigned long data, BUFFER *e
       {
         FREE (&((*tmp)->path));
         tmp1=(*tmp)->next;
-        FREE (tmp);
+        FREE (tmp);		/* __FREE_CHECKED__ */
         *tmp=tmp1;
       }
       continue;
@@ -286,6 +286,8 @@ int mutt_buffy_check (int force)
   BuffyNotify = 0;
 
 #ifdef USE_IMAP
+  BuffyCount += imap_buffy_check (force);
+
   if (!Context || Context->magic != M_IMAP)
 #endif
 #ifdef USE_POP
@@ -300,12 +302,14 @@ int mutt_buffy_check (int force)
   
   for (tmp = Incoming; tmp; tmp = tmp->next)
   {
+#ifdef USE_IMAP
+    if (tmp->magic != M_IMAP)
+#endif
     tmp->new = 0;
 
 #ifdef USE_IMAP
-    if (mx_is_imap (tmp->path))
-      tmp->magic = M_IMAP;
-    else
+    if (tmp->magic != M_IMAP)
+    {
 #endif
 #ifdef USE_POP
     if (mx_is_pop (tmp->path))
@@ -324,6 +328,9 @@ int mutt_buffy_check (int force)
 #endif
       continue;
     }
+#ifdef USE_IMAP
+    }
+#endif
 
     /* check to see if the folder is the currently selected folder
      * before polling */
@@ -398,21 +405,6 @@ int mutt_buffy_check (int force)
 	if ((tmp->new = mh_buffy (tmp->path)) > 0)
 	  BuffyCount++;
 	break;
-	
-#ifdef USE_IMAP
-      case M_IMAP:
-	if ((tmp->new = imap_mailbox_check (tmp->path, 1)) > 0)
-	  BuffyCount++;
-	else
-	  tmp->new = 0;
-
-	break;
-#endif
-
-#ifdef USE_POP
-      case M_POP:
-	break;
-#endif
       }
     }
 #ifdef BUFFY_SIZE
