@@ -15,8 +15,8 @@
  *
  *     You should have received a copy of the GNU General Public
  *     License along with this program; if not, write to the Free
- *     Software Foundation, Inc., 59 Temple Place - Suite 330,
- *     Boston, MA  02111, USA.
+ *     Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ *     Boston, MA  02110-1301, USA.
  */
 
 #if HAVE_CONFIG_H
@@ -450,7 +450,7 @@ int mutt_convert_string (char **ps, const char *from, const char *to, int flags)
 
     *ob = '\0';
 
-    FREE (ps);
+    FREE (ps);		/* __FREE_CHECKED__ */
     *ps = buf;
     
     mutt_str_adjust (ps);
@@ -589,5 +589,5 @@ void fgetconv_close (FGETCONV **_fc)
 
   if (fc->cd != (iconv_t)-1)
     iconv_close (fc->cd);
-  FREE (_fc);
+  FREE (_fc);		/* __FREE_CHECKED__ */
 }
