@@ -14,7 +14,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 /*
@@ -87,7 +87,7 @@ void crypt_init (void)
       crypto_module_register (&crypt_mod_smime_gpgme);
 #else
       mutt_message (_("\"crypt_use_gpgme\" set"
-                      " but not build with GPGME support."));
+                      " but not built with GPGME support."));
       if (mutt_any_key_to_continue (NULL) == -1)
 	mutt_exit(1);
 #endif
