@@ -13,7 +13,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */ 
 
 #if HAVE_CONFIG_H
@@ -99,6 +99,8 @@ static const char *No_visible = N_("No visible messages.");
 			mutt_error ("Operation not permitted by the IMAP ACL for this mailbox"); \
 			break; \
 		}
+#else
+#define CHECK_IMAP_ACL(aclbit) /**/
 #endif
 
 #define CHECK_ATTACH if(option(OPTATTACHMSG)) \
@@ -112,7 +114,6 @@ static const char *No_visible = N_("No visible messages.");
 #define OLDHDR Context->hdrs[Context->v2r[menu->oldcurrent]]
 #define UNREAD(h) mutt_thread_contains_unread (Context, h)
 
-extern const char *ReleaseDate;
 extern size_t UngetCount;
 
 void index_make_entry (char *s, size_t l, MUTTMENU *menu, int num)
@@ -589,10 +590,6 @@ int mutt_index_menu (void)
 	move (menu->current - menu->top + menu->offset, COLS - 1);
       mutt_refresh ();
 
-      op = km_dokey (MENU_MAIN);
-
-      dprint(4, (debugfile, "mutt_index_menu[%d]: Got op %d\n", __LINE__, op));
-
 #if defined (USE_SLANG_CURSES) || defined (HAVE_RESIZETERM)
       if (SigWinch)
       {
@@ -611,6 +608,10 @@ int mutt_index_menu (void)
       }
 #endif
 
+      op = km_dokey (MENU_MAIN);
+
+      dprint(4, (debugfile, "mutt_index_menu[%d]: Got op %d\n", __LINE__, op));
+
       if (op == -1)
 	continue; /* either user abort or timeout */
       
@@ -804,10 +805,7 @@ int mutt_index_menu (void)
 	CHECK_MSGCOUNT;
         CHECK_VISIBLE;
 	CHECK_READONLY;
-
-#ifdef USE_IMAP
-CHECK_IMAP_ACL(IMAP_ACL_DELETE);
-#endif
+	CHECK_IMAP_ACL(IMAP_ACL_DELETE);
 
 	CHECK_ATTACH;
 	mutt_pattern_func (M_DELETE, _("Delete messages matching: "));
@@ -868,6 +866,8 @@ CHECK_IMAP_ACL(IMAP_ACL_DELETE);
 	    mutt_draw_tree (Context);
 	  menu->redraw = REDRAW_FULL;
 	}
+        if (Context->pattern)
+	  mutt_message _("To view all messages, limit to \"all\".");
 	break;	  
 
       case OP_QUIT:
@@ -972,10 +972,7 @@ CHECK_IMAP_ACL(IMAP_ACL_DELETE);
 	CHECK_MSGCOUNT;
         CHECK_VISIBLE;
 	CHECK_READONLY;
-
-#ifdef USE_IMAP
-CHECK_IMAP_ACL(IMAP_ACL_DELETE);
-#endif
+	CHECK_IMAP_ACL(IMAP_ACL_DELETE);
 
 	if (mutt_pattern_func (M_UNDELETE, _("Undelete messages matching: ")) == 0)
 	  menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
@@ -995,7 +992,7 @@ CHECK_IMAP_ACL(IMAP_ACL_DELETE);
 
 #ifdef USE_IMAP
       case OP_MAIN_IMAP_FETCH:
-	if (Context->magic == M_IMAP)
+	if (Context && Context->magic == M_IMAP)
 	  imap_check_mailbox (Context, &index_hint, 1);
         break;
 #endif
@@ -1072,7 +1069,15 @@ CHECK_IMAP_ACL(IMAP_ACL_DELETE);
 	mutt_buffy (buf, sizeof (buf));
 
 	if (mutt_enter_fname (cp, buf, sizeof (buf), &menu->redraw, 1) == -1)
-	  break;
+        {
+          if (menu->menu == MENU_PAGER)
+          {
+            op = OP_DISPLAY_MESSAGE;
+            continue;
+          }
+          else
+            break;
+        }
 	if (!buf[0])
 	{
 	  CLEARLINE (LINES-1);
@@ -1225,6 +1230,7 @@ CHECK_IMAP_ACL(IMAP_ACL_DELETE);
 	CHECK_MSGCOUNT;
         CHECK_VISIBLE;
 	CHECK_READONLY;
+	CHECK_IMAP_ACL(IMAP_ACL_DELETE);
 
         if ((Sort & SORT_MASK) != SORT_THREADS)
 	  mutt_error _("Threading is not enabled.");
@@ -1492,6 +1498,7 @@ CHECK_IMAP_ACL(IMAP_ACL_DELETE);
 	CHECK_MSGCOUNT;
         CHECK_VISIBLE;
 	CHECK_READONLY;
+	CHECK_IMAP_ACL(IMAP_ACL_WRITE);
 
 #ifdef USE_POP
 	if (Context->magic == M_POP)
@@ -1502,10 +1509,6 @@ CHECK_IMAP_ACL(IMAP_ACL_DELETE);
 	}
 #endif
 
-#ifdef USE_IMAP
-CHECK_IMAP_ACL(IMAP_ACL_WRITE);
-#endif
-
         if (tag)
         {
 	  for (j = 0; j < Context->vcount; j++)
@@ -1541,10 +1544,7 @@ CHECK_IMAP_ACL(IMAP_ACL_WRITE);
 	CHECK_MSGCOUNT;
         CHECK_VISIBLE;
 	CHECK_READONLY;
-
-#ifdef USE_IMAP
-CHECK_IMAP_ACL(IMAP_ACL_SEEN);
-#endif
+	CHECK_IMAP_ACL(IMAP_ACL_SEEN);
 
 	if (tag)
 	{
@@ -1658,10 +1658,7 @@ CHECK_IMAP_ACL(IMAP_ACL_SEEN);
 	CHECK_MSGCOUNT;
         CHECK_VISIBLE;
 	CHECK_READONLY;
-	
-/* #ifdef USE_IMAP
-CHECK_IMAP_ACL(IMAP_ACL_WRITE);
-#endif */
+	/* CHECK_IMAP_ACL(IMAP_ACL_WRITE); */
 
 	if (mutt_change_flag (tag ? NULL : CURHDR, (op == OP_MAIN_SET_FLAG)) == 0)
 	{
@@ -1801,10 +1798,7 @@ CHECK_IMAP_ACL(IMAP_ACL_WRITE);
 	CHECK_MSGCOUNT;
         CHECK_VISIBLE;
 	CHECK_READONLY;
-	
-#ifdef USE_IMAP
-CHECK_IMAP_ACL(IMAP_ACL_DELETE);
-#endif
+	CHECK_IMAP_ACL(IMAP_ACL_DELETE);
 
 	if (tag)
 	{
@@ -1845,10 +1839,7 @@ CHECK_IMAP_ACL(IMAP_ACL_DELETE);
 	CHECK_MSGCOUNT;
         CHECK_VISIBLE;
 	CHECK_READONLY;
-
-#ifdef USE_IMAP
-CHECK_IMAP_ACL(IMAP_ACL_DELETE);
-#endif
+	CHECK_IMAP_ACL(IMAP_ACL_DELETE);
 
 	rc = mutt_thread_set_flag (CURHDR, M_DELETE, 1,
 				   op == OP_DELETE_THREAD ? 0 : 1);
@@ -1889,6 +1880,7 @@ CHECK_IMAP_ACL(IMAP_ACL_DELETE);
         CHECK_VISIBLE;
 	CHECK_READONLY;
 	CHECK_ATTACH;
+	CHECK_IMAP_ACL(IMAP_ACL_INSERT);
 
 #ifdef USE_POP
 	if (Context->magic == M_POP)
@@ -1899,10 +1891,6 @@ CHECK_IMAP_ACL(IMAP_ACL_DELETE);
 	}
 #endif
 
-#ifdef USE_IMAP
-CHECK_IMAP_ACL(IMAP_ACL_INSERT);
-#endif
-
 	if (option (OPTPGPAUTODEC) && (tag || !(CURHDR->security & PGP_TRADITIONAL_CHECKED))) 
 	  mutt_check_traditional_pgp (tag ? NULL : CURHDR, &menu->redraw);
         mutt_edit_message (Context, tag ? NULL : CURHDR);
@@ -1988,32 +1976,51 @@ CHECK_IMAP_ACL(IMAP_ACL_INSERT);
 	  continue;
 	}
         break;
-      
+
       case OP_PIPE:
 
 	CHECK_MSGCOUNT;
-        CHECK_VISIBLE;
+	CHECK_VISIBLE;
 	mutt_pipe_message (tag ? NULL : CURHDR);
+
+#ifdef USE_IMAP
+	/* in an IMAP folder index with imap_peek=no, piping could change
+	 * new or old messages status to read. Redraw what's needed.
+	 */
+	if (Context->magic == M_IMAP && !option (OPTIMAPPEEK))
+	{
+	  menu->redraw = (tag ? REDRAW_INDEX : REDRAW_CURRENT) | REDRAW_STATUS;
+	}
+#endif
+
 	MAYBE_REDRAW (menu->redraw);
 	break;
 
       case OP_PRINT:
 
 	CHECK_MSGCOUNT;
-        CHECK_VISIBLE;
+	CHECK_VISIBLE;
 	mutt_print_message (tag ? NULL : CURHDR);
+
+#ifdef USE_IMAP
+	/* in an IMAP folder index with imap_peek=no, printing could change
+	 * new or old messages status to read. Redraw what's needed.
+	 */
+	if (Context->magic == M_IMAP && !option (OPTIMAPPEEK))
+	{
+	  menu->redraw = (tag ? REDRAW_INDEX : REDRAW_CURRENT) | REDRAW_STATUS;
+	}
+#endif
+
 	break;
 
       case OP_MAIN_READ_THREAD:
       case OP_MAIN_READ_SUBTHREAD:
 
 	CHECK_MSGCOUNT;
-        CHECK_VISIBLE;
+	CHECK_VISIBLE;
 	CHECK_READONLY;
-
-#ifdef USE_IMAP
-CHECK_IMAP_ACL(IMAP_ACL_SEEN);
-#endif
+	CHECK_IMAP_ACL(IMAP_ACL_SEEN);
 
 	rc = mutt_thread_set_flag (CURHDR, M_READ, 1,
 				   op == OP_MAIN_READ_THREAD ? 0 : 1);
@@ -2100,10 +2107,7 @@ CHECK_IMAP_ACL(IMAP_ACL_SEEN);
 	CHECK_MSGCOUNT;
         CHECK_VISIBLE;
 	CHECK_READONLY;
-
-#ifdef USE_IMAP
-CHECK_IMAP_ACL(IMAP_ACL_DELETE);
-#endif
+	CHECK_IMAP_ACL(IMAP_ACL_DELETE);
 	
 	if (tag)
 	{
@@ -2130,10 +2134,7 @@ CHECK_IMAP_ACL(IMAP_ACL_DELETE);
 	CHECK_MSGCOUNT;
         CHECK_VISIBLE;
 	CHECK_READONLY;
-
-#ifdef USE_IMAP
-CHECK_IMAP_ACL(IMAP_ACL_DELETE);
-#endif
+	CHECK_IMAP_ACL(IMAP_ACL_DELETE);
 
 	rc = mutt_thread_set_flag (CURHDR, M_DELETE, 0,
 				   op == OP_UNDELETE_THREAD ? 0 : 1);
