@@ -15,14 +15,20 @@
  *
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
+/* this comment bumps Id because of the attachment counting patch. */
+
 #if HAVE_CONFIG_H
 #include "config.h"
 #endif				/* HAVE_CONFIG_H */
 
-#if HAVE_GDBM
+#if HAVE_QDBM
+#include <depot.h>
+#include <cabin.h>
+#include <villa.h>
+#elif HAVE_GDBM
 #include <gdbm.h>
 #elif HAVE_DB4
 #include <db.h>
@@ -34,6 +40,7 @@
 #include <sys/time.h>
 #endif
 #include "mutt.h"
+#include "hcache.h"
 #ifdef USE_IMAP
 #include "message.h"
 #endif
@@ -42,7 +49,14 @@
 #include "lib.h"
 #include "md5.h"
 
-#if HAVE_GDBM
+#if HAVE_QDBM
+static struct header_cache
+{
+  VILLA *db;
+  char *folder;
+  unsigned int crc;
+} HEADER_CACHE;
+#elif HAVE_GDBM
 static struct header_cache
 {
   GDBM_FILE db;
@@ -58,6 +72,9 @@ static struct header_cache
   int fd;
   char lockfile[_POSIX_PATH_MAX];
 } HEADER_CACHE;
+
+static void mutt_hcache_dbt_init(DBT * dbt, void *data, size_t len);
+static void mutt_hcache_dbt_empty_init(DBT * dbt);
 #endif
 
 typedef union
@@ -450,7 +467,7 @@ generate_crc32()
   SPAM_LIST *sp = SpamList;
   RX_LIST *rx = NoSpamList;
 
-  crc = crc32(crc, (unsigned char const *) "$Id: hcache.c,v 3.7 2005/07/06 21:03:00 brendan Exp $", mutt_strlen("$Id: hcache.c,v 3.7 2005/07/06 21:03:00 brendan Exp $"));
+  crc = crc32(crc, (unsigned char const *) "$Id: hcache.c,v 3.17 2006/05/18 18:35:10 brendan Exp $", mutt_strlen("$Id: hcache.c,v 3.17 2006/05/18 18:35:10 brendan Exp $"));
 
 #if HAVE_LANGINFO_CODESET
   crc = crc32(crc, (unsigned char const *) Charset, mutt_strlen(Charset));
@@ -549,10 +566,9 @@ mutt_hcache_per_folder(const char *path, const char *folder)
 /* This function transforms a header into a char so that it is useable by
  * db_store */
 static void *
-mutt_hcache_dump(void *_db, HEADER * h, int *off,
+mutt_hcache_dump(header_cache_t *h, HEADER * header, int *off,
 		 unsigned long uid_validity)
 {
-  struct header_cache *db = _db;
   unsigned char *d = NULL;
   *off = 0;
 
@@ -568,15 +584,15 @@ mutt_hcache_dump(void *_db, HEADER * h, int *off,
   }
   *off += sizeof (validate);
 
-  d = dump_int(db->crc, d, off);
+  d = dump_int(h->crc, d, off);
 
   lazy_realloc(&d, *off + sizeof (HEADER));
-  memcpy(d + *off, h, sizeof (HEADER));
+  memcpy(d + *off, header, sizeof (HEADER));
   *off += sizeof (HEADER);
 
-  d = dump_envelope(h->env, d, off);
-  d = dump_body(h->content, d, off);
-  d = dump_char(h->maildir_flags, d, off);
+  d = dump_envelope(header->env, d, off);
+  d = dump_body(header->content, d, off);
+  d = dump_char(header->maildir_flags, d, off);
 
   return d;
 }
@@ -615,9 +631,210 @@ mutt_hcache_restore(const unsigned char *d, HEADER ** oh)
   return h;
 }
 
-#if HAVE_GDBM
+void *
+mutt_hcache_fetch(header_cache_t *h, const char *filename,
+		  size_t(*keylen) (const char *fn))
+{
+  void* data;
+
+  data = mutt_hcache_fetch_raw (h, filename, keylen);
+
+  if (!data || !crc32_matches(data, h->crc))
+  {
+    FREE(&data);
+    return NULL;
+  }
+  
+  return data;
+}
 
 void *
+mutt_hcache_fetch_raw (header_cache_t *h, const char *filename,
+                       size_t(*keylen) (const char *fn))
+{
+#ifndef HAVE_DB4
+  char path[_POSIX_PATH_MAX];
+  int ksize;
+#endif
+#ifdef HAVE_QDBM
+  char *data = NULL;
+#elif HAVE_GDBM
+  datum key;
+  datum data;
+#elif HAVE_DB4
+  DBT key;
+  DBT data;
+#endif
+  
+  if (!h)
+    return NULL;
+  
+#ifdef HAVE_DB4
+  filename++;			/* skip '/' */
+
+  mutt_hcache_dbt_init(&key, (void *) filename, keylen(filename));
+  mutt_hcache_dbt_empty_init(&data);
+  data.flags = DB_DBT_MALLOC;
+  
+  h->db->get(h->db, NULL, &key, &data, 0);
+  
+  return data.data;
+#else
+  strncpy(path, h->folder, sizeof (path));
+  safe_strcat(path, sizeof (path), filename);
+
+  ksize = strlen (h->folder) + keylen (path + strlen (h->folder));  
+#endif
+#ifdef HAVE_QDBM
+  data = vlget(h->db, path, ksize, NULL);
+  
+  return data;
+#elif HAVE_GDBM
+  key.dptr = path;
+  key.dsize = ksize;
+  
+  data = gdbm_fetch(h->db, key);
+  
+  return data.dptr;
+#endif
+}
+
+int
+mutt_hcache_store(header_cache_t *h, const char *filename, HEADER * header,
+		  unsigned long uid_validity,
+		  size_t(*keylen) (const char *fn))
+{
+  char* data;
+  int dlen;
+  int ret;
+  
+  if (!h)
+    return -1;
+  
+  data = mutt_hcache_dump(h, header, &dlen, uid_validity);
+  ret = mutt_hcache_store_raw (h, filename, data, dlen, keylen);
+  
+  FREE(&data);
+  
+  return ret;
+}
+
+int
+mutt_hcache_store_raw (header_cache_t* h, const char* filename, void* data,
+                       size_t dlen, size_t(*keylen) (const char* fn))
+{
+#ifndef HAVE_DB4
+  char path[_POSIX_PATH_MAX];
+  int ksize;
+#endif
+#if HAVE_GDBM
+  datum key;
+  datum databuf;
+#elif HAVE_DB4
+  DBT key;
+  DBT databuf;
+#endif
+  
+  if (!h)
+    return -1;
+
+#if HAVE_DB4
+  filename++;			/* skip '/' */
+  
+  mutt_hcache_dbt_init(&key, (void *) filename, keylen(filename));
+  
+  mutt_hcache_dbt_empty_init(&databuf);
+  databuf.flags = DB_DBT_USERMEM;
+  databuf.data = data;
+  databuf.size = dlen;
+  databuf.ulen = dlen;
+  
+  return h->db->put(h->db, NULL, &key, &databuf, 0);
+#else
+  strncpy(path, h->folder, sizeof (path));
+  safe_strcat(path, sizeof (path), filename);
+
+  ksize = strlen(h->folder) + keylen(path + strlen(h->folder));
+#endif
+#if HAVE_QDBM
+  return vlput(h->db, path, ksize, data, dlen, VL_DOVER);
+#elif HAVE_GDBM
+  key.dptr = path;
+  key.dsize = ksize;
+  
+  databuf.dsize = dlen;
+  databuf.dptr = data;
+  
+  return gdbm_store(h->db, key, databuf, GDBM_REPLACE);
+#endif
+}
+
+#if HAVE_QDBM
+header_cache_t *
+mutt_hcache_open(const char *path, const char *folder)
+{
+  struct header_cache *h = safe_calloc(1, sizeof (HEADER_CACHE));
+  int    flags = VL_OWRITER | VL_OCREAT;
+  h->db = NULL;
+  h->folder = safe_strdup(folder);
+  h->crc = generate_crc32();
+
+  if (!path || path[0] == '\0')
+  {
+    FREE(&h->folder);
+    FREE(&h);
+    return NULL;
+  }
+
+  path = mutt_hcache_per_folder(path, folder);
+
+  if (option(OPTHCACHECOMPRESS))
+    flags |= VL_OZCOMP;
+
+  h->db = vlopen(path, flags, VL_CMPLEX);
+  if (h->db)
+    return h;
+  else
+  {
+    FREE(&h->folder);
+    FREE(&h);
+
+    return NULL;
+  }
+}
+
+void
+mutt_hcache_close(header_cache_t *h)
+{
+  if (!h)
+    return;
+
+  vlclose(h->db);
+  FREE(&h->folder);
+  FREE(&h);
+}
+
+int
+mutt_hcache_delete(header_cache_t *h, const char *filename,
+		   size_t(*keylen) (const char *fn))
+{
+  char path[_POSIX_PATH_MAX];
+  int ksize;
+
+  if (!h)
+    return -1;
+
+  strncpy(path, h->folder, sizeof (path));
+  safe_strcat(path, sizeof (path), filename);
+
+  ksize = strlen(h->folder) + keylen(path + strlen(h->folder));
+
+  return vlout(h->db, path, ksize);
+}
+
+#elif HAVE_GDBM
+
+header_cache_t *
 mutt_hcache_open(const char *path, const char *folder)
 {
   struct header_cache *h = safe_calloc(1, sizeof (HEADER_CACHE));
@@ -653,10 +870,8 @@ mutt_hcache_open(const char *path, const char *folder)
 }
 
 void
-mutt_hcache_close(void *db)
+mutt_hcache_close(header_cache_t *h)
 {
-  struct header_cache *h = db;
-
   if (!h)
     return;
 
@@ -665,70 +880,11 @@ mutt_hcache_close(void *db)
   FREE(&h);
 }
 
-void *
-mutt_hcache_fetch(void *db, const char *filename,
-		  size_t(*keylen) (const char *fn))
-{
-  struct header_cache *h = db;
-  datum key;
-  datum data;
-  char path[_POSIX_PATH_MAX];
-
-  if (!h)
-    return NULL;
-
-  strncpy(path, h->folder, sizeof (path));
-  safe_strcat(path, sizeof (path), filename);
-
-  key.dptr = path;
-  key.dsize = strlen(h->folder) + keylen(path + strlen(h->folder));
-
-  data = gdbm_fetch(h->db, key);
-
-  if (!crc32_matches(data.dptr, h->crc))
-  {
-    FREE(&data.dptr);
-    return NULL;
-  }
-
-  return data.dptr;
-}
-
 int
-mutt_hcache_store(void *db, const char *filename, HEADER * header,
-		  unsigned long uid_validity,
-		  size_t(*keylen) (const char *fn))
-{
-  struct header_cache *h = db;
-  datum key;
-  datum data;
-  char path[_POSIX_PATH_MAX];
-  int ret;
-
-  if (!h)
-    return -1;
-
-  strncpy(path, h->folder, sizeof (path));
-  safe_strcat(path, sizeof (path), filename);
-
-  key.dptr = path;
-  key.dsize = strlen(h->folder) + keylen(path + strlen(h->folder));
-
-  data.dptr = mutt_hcache_dump(db, header, &data.dsize, uid_validity);
-
-  ret = gdbm_store(h->db, key, data, GDBM_REPLACE);
-
-  FREE(&data.dptr);
-
-  return ret;
-}
-
-int
-mutt_hcache_delete(void *db, const char *filename,
+mutt_hcache_delete(header_cache_t *h, const char *filename,
 		   size_t(*keylen) (const char *fn))
 {
   datum key;
-  struct header_cache *h = db;
   char path[_POSIX_PATH_MAX];
 
   if (!h)
@@ -761,7 +917,7 @@ mutt_hcache_dbt_empty_init(DBT * dbt)
   dbt->flags = 0;
 }
 
-void *
+header_cache_t *
 mutt_hcache_open(const char *path, const char *folder)
 {
   struct stat sb;
@@ -841,10 +997,8 @@ mutt_hcache_open(const char *path, const char *folder)
 }
 
 void
-mutt_hcache_close(void *db)
+mutt_hcache_close(header_cache_t *h)
 {
-  struct header_cache *h = db;
-
   if (!h)
     return;
 
@@ -855,70 +1009,11 @@ mutt_hcache_close(void *db)
   FREE(&h);
 }
 
-void *
-mutt_hcache_fetch(void *db, const char *filename,
-		  size_t(*keylen) (const char *fn))
-{
-  DBT key;
-  DBT data;
-  struct header_cache *h = db;
-
-  if (!h)
-    return NULL;
-
-  filename++;			/* skip '/' */
-
-  mutt_hcache_dbt_init(&key, (void *) filename, keylen(filename));
-  mutt_hcache_dbt_empty_init(&data);
-  data.flags = DB_DBT_MALLOC;
-
-  h->db->get(h->db, NULL, &key, &data, 0);
-
-  if (!crc32_matches(data.data, h->crc))
-  {
-    FREE(&data.data);
-    return NULL;
-  }
-
-  return data.data;
-}
-
 int
-mutt_hcache_store(void *db, const char *filename, HEADER * header,
-		  unsigned long uid_validity,
-		  size_t(*keylen) (const char *fn))
-{
-  DBT key;
-  DBT data;
-  int ret;
-  struct header_cache *h = db;
-
-  if (!h)
-    return -1;
-
-  filename++;			/* skip '/' */
-
-  mutt_hcache_dbt_init(&key, (void *) filename, keylen(filename));
-
-  mutt_hcache_dbt_empty_init(&data);
-  data.flags = DB_DBT_USERMEM;
-  data.data = mutt_hcache_dump(db, header, (signed int *) &data.size,
-	      uid_validity);
-  data.ulen = data.size;
-
-  ret = h->db->put(h->db, NULL, &key, &data, 0);
-
-  FREE(&data.data);
-
-  return ret;
-}
-
-int
-mutt_hcache_delete(void *db, const char *filename,
+mutt_hcache_delete(header_cache_t *h, const char *filename,
 		   size_t(*keylen) (const char *fn))
 {
   DBT key;
-  struct header_cache *h = db;
 
   if (!h)
     return -1;
