@@ -13,7 +13,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */ 
 
 #if HAVE_CONFIG_H
@@ -81,7 +81,12 @@ void mutt_history_add (history_class_t hclass, const char *s)
   {
     prev = h->last - 1;
     if (prev < 0) prev = HistSize - 1;
-    if (!h->hist[prev] || mutt_strcmp (h->hist[prev], s) != 0)
+
+    /* don't add to prompt history:
+     *  - lines beginning by a space
+     *  - repeated lines
+     */
+    if (*s != ' ' && (!h->hist[prev] || mutt_strcmp (h->hist[prev], s) != 0))
     {
       mutt_str_replace (&h->hist[h->last++], s);
       if (h->last > HistSize - 1)
