@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>, and others
+ * Copyright (C) 1996-2002 Michael R. Elkins <me@mutt.org>, and others
  *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,10 +13,16 @@
  *
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "mutt.h"
+#include "mailbox.h"
+#include "mutt_crypt.h"
 
 #include <limits.h>
 #include <string.h>
@@ -35,6 +41,8 @@ typedef struct hook
 
 static HOOK *Hooks = NULL;
 
+static int current_hook_type = 0;
+
 int mutt_parse_hook (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
 {
   HOOK *ptr;
@@ -62,7 +70,7 @@ int mutt_parse_hook (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
     goto error;
   }
 
-  mutt_extract_token (&command, s, (data & (M_FOLDERHOOK | M_SENDHOOK)) ?  M_TOKEN_SPACE : 0);
+  mutt_extract_token (&command, s, (data & (M_FOLDERHOOK | M_SENDHOOK | M_SEND2HOOK | M_ACCOUNTHOOK | M_REPLYHOOK)) ?  M_TOKEN_SPACE : 0);
 
   if (!command.data)
   {
@@ -84,10 +92,8 @@ int mutt_parse_hook (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
     memset (&pattern, 0, sizeof (pattern));
     pattern.data = safe_strdup (path);
   }
-  else if (DefaultHook && !(data & M_CHARSETHOOK)
-#ifdef HAVE_PGP
-      && !(data & M_PGPHOOK)
-#endif /* HAVE_PGP */
+  else if (DefaultHook && !(data & (M_CHARSETHOOK | M_ACCOUNTHOOK))
+           && (!WithCrypto || !(data & M_CRYPTHOOK))
       )
   {
     char tmp[HUGE_STRING];
@@ -115,11 +121,11 @@ int mutt_parse_hook (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
 	ptr->rx.not == not &&
 	!mutt_strcmp (pattern.data, ptr->rx.pattern))
     {
-      if (data & (M_FOLDERHOOK | M_SENDHOOK))
+      if (data & (M_FOLDERHOOK | M_SENDHOOK | M_SEND2HOOK | M_MESSAGEHOOK | M_ACCOUNTHOOK | M_REPLYHOOK))
       {
-	/* folder-hook and send-hook allow multiple commands with the same
-	   pattern, so if we've already seen this pattern/command pair, just
-	   ignore it instead of creating a duplicate */
+	/* these hooks allow multiple commands with the same
+	 * pattern, so if we've already seen this pattern/command pair, just
+	 * ignore it instead of creating a duplicate */
 	if (!mutt_strcmp (ptr->command, command.data))
 	{
 	  FREE (&command.data);
@@ -130,10 +136,10 @@ int mutt_parse_hook (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
       else
       {
 	/* other hooks only allow one command per pattern, so update the
-	   entry with the new command.  this currently does not change the
-	   order of execution of the hooks, which i think is desirable since
-	   a common action to perform is to change the default (.) entry
-	   based upon some other information. */
+	 * entry with the new command.  this currently does not change the
+	 * order of execution of the hooks, which i think is desirable since
+	 * a common action to perform is to change the default (.) entry
+	 * based upon some other information. */
 	FREE (&ptr->command);
 	ptr->command = command.data;
 	FREE (&pattern.data);
@@ -144,23 +150,25 @@ int mutt_parse_hook (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
       break;
   }
 
-  if (data & (M_SENDHOOK | M_SAVEHOOK | M_FCCHOOK))
+  if (data & (M_SENDHOOK | M_SEND2HOOK | M_SAVEHOOK | M_FCCHOOK | M_MESSAGEHOOK | M_REPLYHOOK))
   {
-    if ((pat = mutt_pattern_comp (pattern.data, (data & (M_SENDHOOK | M_FCCHOOK)) ? 0 : M_FULL_MSG, err)) == NULL)
+    if ((pat = mutt_pattern_comp (pattern.data,
+	   (data & (M_SENDHOOK | M_SEND2HOOK | M_FCCHOOK | M_REPLYHOOK)) ? 0 : M_FULL_MSG,
+				  err)) == NULL)
       goto error;
   }
   else
   {
     rx = safe_malloc (sizeof (regex_t));
-#ifdef M_PGPHOOK
-    if ((rc = REGCOMP (rx, pattern.data, ((data & (M_PGPHOOK|M_CHARSETHOOK)) ? REG_ICASE : 0))) != 0)
+#ifdef M_CRYPTHOOK
+    if ((rc = REGCOMP (rx, NONULL(pattern.data), ((data & (M_CRYPTHOOK|M_CHARSETHOOK)) ? REG_ICASE : 0))) != 0)
 #else
-      if ((rc = REGCOMP (rx, pattern.data, (data & M_CHARSETHOOK) ? REG_ICASE : 0)) != 0)
-#endif /* HAVE_PGP */
+    if ((rc = REGCOMP (rx, NONULL(pattern.data), (data & (M_CHARSETHOOK|M_ICONVHOOK)) ? REG_ICASE : 0)) != 0)
+#endif /* M_CRYPTHOOK */
     {
       regerror (rc, rx, err->data, err->dsize);
       regfree (rx);
-      safe_free ((void **) &rx);
+      FREE (&rx);
       goto error;
     }
   }
@@ -231,18 +239,33 @@ int mutt_parse_unhook (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
   {
     mutt_extract_token (buf, s, 0);
     if (mutt_strcmp ("*", buf->data) == 0)
+    {
+      if (current_hook_type)
+      {
+	snprintf (err->data, err->dsize,
+		  _("unhook: Can't do unhook * from within a hook."));
+	return -1;
+      }
       delete_hooks (0);
+    }
     else
     {
       int type = mutt_get_hook_type (buf->data);
-      if (type)
-	delete_hooks (type);
-      else
+
+      if (!type)
       {
 	snprintf (err->data, err->dsize,
 		 _("unhook: unknown hook type: %s"), buf->data);
 	return (-1);
       }
+      if (current_hook_type == type)
+      {
+	snprintf (err->data, err->dsize,
+		  _("unhook: Can't delete a %s from within a %s."),
+		  buf->data, buf->data);
+	return -1;
+      }
+      delete_hooks (type);
     }
   }
   return 0;
@@ -254,6 +277,8 @@ void mutt_folder_hook (char *path)
   BUFFER err, token;
   char buf[STRING];
 
+  current_hook_type = M_FOLDERHOOK;
+  
   err.data = buf;
   err.dsize = sizeof (buf);
   memset (&token, 0, sizeof (token));
@@ -270,13 +295,16 @@ void mutt_folder_hook (char *path)
 	{
 	  mutt_error ("%s", err.data);
 	  FREE (&token.data);
-	  sleep (1);	/* pause a moment to let the user see the error */
+	  mutt_sleep (1);	/* pause a moment to let the user see the error */
+	  current_hook_type = 0;
 	  return;
 	}
       }
     }
   }
   FREE (&token.data);
+  
+  current_hook_type = 0;
 }
 
 char *mutt_find_hook (int type, const char *pat)
@@ -292,12 +320,14 @@ char *mutt_find_hook (int type, const char *pat)
   return (NULL);
 }
 
-void mutt_send_hook (HEADER *hdr)
+void mutt_message_hook (CONTEXT *ctx, HEADER *hdr, int type)
 {
   BUFFER err, token;
   HOOK *hook;
   char buf[STRING];
 
+  current_hook_type = type;
+  
   err.data = buf;
   err.dsize = sizeof (buf);
   memset (&token, 0, sizeof (token));
@@ -306,17 +336,19 @@ void mutt_send_hook (HEADER *hdr)
     if(!hook->command)
       continue;
 
-    if (hook->type & M_SENDHOOK)
-      if ((mutt_pattern_exec (hook->pattern, 0, NULL, hdr) > 0) ^ hook->rx.not)
+    if (hook->type & type)
+      if ((mutt_pattern_exec (hook->pattern, 0, ctx, hdr) > 0) ^ hook->rx.not)
 	if (mutt_parse_rc_line (hook->command, &token, &err) != 0)
 	{
 	  FREE (&token.data);
 	  mutt_error ("%s", err.data);
-	  sleep (1);
+	  mutt_sleep (1);
+	  current_hook_type = 0;
 	  return;
 	}
   }
   FREE (&token.data);
+  current_hook_type = 0;
 }
 
 static int
@@ -382,8 +414,8 @@ void mutt_select_fcc (char *path, size_t pathlen, HEADER *hdr)
     {
       adr = env->to ? env->to : (env->cc ? env->cc : env->bcc);
       mutt_safe_path (buf, sizeof (buf), adr);
-      snprintf (path, pathlen, "%s/%s", NONULL (Maildir), buf);
-      if (!option (OPTFORCENAME) && access (path, W_OK) != 0)
+      mutt_concat_path (path, NONULL(Maildir), buf, pathlen);
+      if (!option (OPTFORCENAME) && mx_access (path, W_OK) != 0)
 	strfcpy (path, NONULL (Outbox), pathlen);
     }
     else
@@ -410,9 +442,46 @@ char *mutt_charset_hook (const char *chs)
   return _mutt_string_hook (chs, M_CHARSETHOOK);
 }
 
-#ifdef HAVE_PGP
-char *mutt_pgp_hook (ADDRESS *adr)
+char *mutt_iconv_hook (const char *chs)
 {
-  return _mutt_string_hook (adr->mailbox, M_PGPHOOK);
+  return _mutt_string_hook (chs, M_ICONVHOOK);
 }
-#endif /* HAVE_PGP */
+
+char *mutt_crypt_hook (ADDRESS *adr)
+{
+  return _mutt_string_hook (adr->mailbox, M_CRYPTHOOK);
+}
+
+#ifdef USE_SOCKET
+void mutt_account_hook (const char* url)
+{
+  HOOK* hook;
+  BUFFER token;
+  BUFFER err;
+  char buf[STRING];
+
+  err.data = buf;
+  err.dsize = sizeof (buf);
+  memset (&token, 0, sizeof (token));
+
+  for (hook = Hooks; hook; hook = hook->next)
+  {
+    if (! (hook->command && (hook->type & M_ACCOUNTHOOK)))
+      continue;
+
+    if ((regexec (hook->rx.rx, url, 0, NULL, 0) == 0) ^ hook->rx.not)
+    {
+      if (mutt_parse_rc_line (hook->command, &token, &err) == -1)
+      {
+	FREE (&token.data);
+	mutt_error ("%s", err.data);
+	mutt_sleep (1);
+
+	return;
+      }
+    }
+  }
+
+  FREE (&token.data);
+}
+#endif
