@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-3 Brendan Cully <brendan@kublai.com>
+ * Copyright (C) 2000-5 Brendan Cully <brendan@kublai.com>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */ 
 
 /* SASL login/authentication code */
@@ -97,6 +97,17 @@ imap_auth_res_t imap_auth_sasl (IMAP_DATA* idata, const char* method)
   mutt_message (_("Authenticating (%s)..."), mech);
 
   snprintf (buf, sizeof (buf), "AUTHENTICATE %s", mech);
+  if (mutt_bit_isset (idata->capabilities, SASL_IR) && client_start)
+  {
+    len = mutt_strlen (buf);
+    buf[len++] = ' ';
+    if (sasl_encode64 (pc, olen, buf + len, sizeof (buf) - len, &olen) != SASL_OK)
+    {
+      dprint (1, (debugfile, "imap_auth_sasl: error base64-encoding client response.\n"));
+      goto bail;
+    }
+    client_start = olen = 0;
+  }
   imap_cmd_start (idata, buf);
   irc = IMAP_CMD_CONTINUE;
 
@@ -119,7 +130,14 @@ imap_auth_res_t imap_auth_sasl (IMAP_DATA* idata, const char* method)
 
     if (irc == IMAP_CMD_RESPOND)
     {
-      if (sasl_decode64 (idata->cmd.buf+2, strlen (idata->cmd.buf+2), buf, LONG_STRING-1, &len) != SASL_OK)
+      /* Exchange incorrectly returns +\r\n instead of + \r\n */
+      if (idata->buf[1] == '\0')
+      {
+	buf[0] = '\0';
+	len = 0;
+      }
+      else if (sasl_decode64 (idata->buf+2, strlen (idata->buf+2), buf,
+			      LONG_STRING-1, &len) != SASL_OK)
       {
 	dprint (1, (debugfile, "imap_auth_sasl: error base64-decoding server response.\n"));
 	goto bail;
@@ -175,7 +193,7 @@ imap_auth_res_t imap_auth_sasl (IMAP_DATA* idata, const char* method)
   if (rc != SASL_OK)
     goto bail;
 
-  if (imap_code (idata->cmd.buf))
+  if (imap_code (idata->buf))
   {
     mutt_sasl_setup_conn (idata->conn, saslconn);
     return IMAP_AUTH_SUCCESS;
