@@ -119,6 +119,8 @@ int imap_cmd_step (IMAP_DATA* idata)
   size_t len = 0;
   int c;
   int rc;
+  int stillrunning = 0;
+  IMAP_COMMAND* cmd;
 
   if (idata->status == IMAP_FATAL)
   {
@@ -167,27 +169,50 @@ int imap_cmd_step (IMAP_DATA* idata)
   idata->lastread = time (NULL);
 
   /* handle untagged messages. The caller still gets its shot afterwards. */
-  if (!ascii_strncmp (idata->buf, "* ", 2) &&
-      cmd_handle_untagged (idata))
+  if ((!ascii_strncmp (idata->buf, "* ", 2)
+       || !ascii_strncmp (imap_next_word (idata->buf), "OK [", 4))
+      && cmd_handle_untagged (idata))
     return IMAP_CMD_BAD;
 
   /* server demands a continuation response from us */
   if (idata->buf[0] == '+')
     return IMAP_CMD_RESPOND;
 
-  /* tagged completion code. TODO: I believe commands will always be completed
-   * in order, but will have to double-check when I have net again */
+  /* look for tagged command completions */
   rc = IMAP_CMD_CONTINUE;
-  if (!ascii_strncmp (idata->buf, idata->cmds[idata->lastcmd].seq, SEQLEN))
+  c = idata->lastcmd;
+  do
   {
-    idata->cmds[idata->lastcmd].state = cmd_status (idata->buf);
-    idata->lastcmd = (idata->lastcmd + 1) % IMAP_PIPELINE_DEPTH;
-    if (idata->lastcmd == idata->nextcmd)
+    cmd = &idata->cmds[c];
+    if (cmd->state == IMAP_CMD_NEW)
     {
-      rc = cmd_status (idata->buf);
-      imap_cmd_finish (idata);
+      if (!ascii_strncmp (idata->buf, cmd->seq, SEQLEN)) {
+	if (!stillrunning)
+	{
+	  /* first command in queue has finished - move queue pointer up */
+	  idata->lastcmd = (idata->lastcmd + 1) % IMAP_PIPELINE_DEPTH;
+	}
+	cmd->state = cmd_status (idata->buf);
+	/* bogus - we don't know which command result to return here. Caller
+	 * should provide a tag. */
+	rc = cmd->state;
+      }
+      else
+	stillrunning++;
     }
+
+    c = (c + 1) % IMAP_PIPELINE_DEPTH;
   }
+  while (c != idata->nextcmd);
+
+  if (stillrunning)
+    rc = IMAP_CMD_CONTINUE;
+  else
+  {
+    dprint (2, (debugfile, "IMAP queue drained\n"));
+    imap_cmd_finish (idata);
+  }
+  
 
   return rc;
 }
@@ -365,6 +390,7 @@ static int cmd_handle_untagged (IMAP_DATA* idata)
   int count;
 
   s = imap_next_word (idata->buf);
+  pn = imap_next_word (s);
 
   if ((idata->state >= IMAP_SELECTED) && isdigit ((unsigned char) *s))
   {
@@ -384,12 +410,10 @@ static int cmd_handle_untagged (IMAP_DATA* idata)
       if ( !(idata->reopen & IMAP_EXPUNGE_PENDING) &&
 	   count < idata->ctx->msgcount)
       {
-	/* something is wrong because the server reported fewer messages
-	 * than we previously saw
-	 */
-	mutt_error _("Fatal error.  Message count is out of sync!");
-	idata->status = IMAP_FATAL;
-	return -1;
+        /* Notes 6.0.3 has a tendency to report fewer messages exist than
+         * it should. */
+	dprint (1, (debugfile, "Message count is out of sync"));
+	return 0;
       }
       /* at least the InterChange server sends EXISTS messages freely,
        * even when there is no new mail */
@@ -417,7 +441,9 @@ static int cmd_handle_untagged (IMAP_DATA* idata)
   else if (ascii_strncasecmp ("CAPABILITY", s, 10) == 0)
     cmd_parse_capability (idata, s);
   else if (!ascii_strncasecmp ("OK [CAPABILITY", s, 14))
-    cmd_parse_capability (idata, imap_next_word (s));
+    cmd_parse_capability (idata, pn);
+  else if (!ascii_strncasecmp ("OK [CAPABILITY", pn, 14))
+    cmd_parse_capability (idata, imap_next_word (pn));
   else if (ascii_strncasecmp ("LIST", s, 4) == 0)
     cmd_parse_list (idata, s);
   else if (ascii_strncasecmp ("LSUB", s, 4) == 0)
@@ -533,7 +559,7 @@ static void cmd_parse_fetch (IMAP_DATA* idata, char* s)
     {
       h = idata->ctx->hdrs[cur];
       
-      if (h->active && h->index+1 == msgno)
+      if (h && h->active && h->index+1 == msgno)
       {
 	dprint (2, (debugfile, "Message UID %d updated\n", HEADER_DATA(h)->uid));
 	break;
@@ -579,8 +605,9 @@ static void cmd_parse_list (IMAP_DATA* idata, char* s)
   IMAP_LIST* list;
   IMAP_LIST lb;
   char delimbuf[5]; /* worst case: "\\"\0 */
+  long litlen;
 
-  if (idata->cmddata)
+  if (idata->cmddata && idata->cmdtype == IMAP_CT_LIST)
     list = (IMAP_LIST*)idata->cmddata;
   else
     list = &lb;
@@ -621,9 +648,22 @@ static void cmd_parse_list (IMAP_DATA* idata, char* s)
 
   /* Name */
   s = imap_next_word (s);
-  imap_unmunge_mbox_name (s);
-  list->name = s;
-  
+  /* Notes often responds with literals here. We need a real tokenizer. */
+  if (!imap_get_literal_count (s, &litlen))
+  {
+    if (imap_cmd_step (idata) != IMAP_CMD_CONTINUE)
+    {
+      idata->status = IMAP_FATAL;
+      return;
+    }
+    list->name = idata->buf;
+  }
+  else
+  {
+    imap_unmunge_mbox_name (s);
+    list->name = s;
+  }
+
   if (list->name[0] == '\0')
   {
     idata->delim = list->delim;
@@ -639,7 +679,7 @@ static void cmd_parse_lsub (IMAP_DATA* idata, char* s)
   ciss_url_t url;
   IMAP_LIST list;
 
-  if (idata->cmddata)
+  if (idata->cmddata && idata->cmdtype == IMAP_CT_LIST)
   {
     /* caller will handle response itself */
     cmd_parse_list (idata, s);
@@ -649,6 +689,7 @@ static void cmd_parse_lsub (IMAP_DATA* idata, char* s)
   if (!option (OPTIMAPCHECKSUBSCRIBED))
     return;
 
+  idata->cmdtype = IMAP_CT_LIST;
   idata->cmddata = &list;
   cmd_parse_list (idata, s);
   idata->cmddata = NULL;
@@ -811,7 +852,7 @@ static void cmd_parse_status (IMAP_DATA* idata, char* s)
               status->messages, status->recent, status->unseen));
 
   /* caller is prepared to handle the result herself */
-  if (idata->cmddata)
+  if (idata->cmddata && idata->cmdtype == IMAP_CT_STATUS)
   {
     memcpy (idata->cmddata, status, sizeof (IMAP_STATUS));
     return;
