@@ -1452,7 +1452,7 @@ int imap_buffy_check (int force)
     {
       /* Send commands to previous server. Sorting the buffy list
        * may prevent some infelicitous interleavings */
-      if (imap_exec (lastdata, NULL, 0) != IMAP_CMD_OK)
+      if (imap_exec (lastdata, NULL, IMAP_CMD_FAIL_OK) == -1)
         dprint (1, (debugfile, "Error polling mailboxes\n"));
 
       lastdata = NULL;
@@ -1473,7 +1473,7 @@ int imap_buffy_check (int force)
       /* pipeline must be full, drain it */
       dprint (2, (debugfile, "IMAP command pipeline full, draining\n"));
 
-      if (imap_exec (idata, NULL, 0) != IMAP_CMD_OK)
+      if (imap_exec (idata, NULL, IMAP_CMD_FAIL_OK) == -1)
         dprint (1, (debugfile, "Error polling mailboxes\n"));
       
       if (imap_cmd_queue (idata, command) < 0) {
@@ -1484,9 +1484,9 @@ int imap_buffy_check (int force)
     }
   }
 
-  if (lastdata && (imap_exec (lastdata, NULL, 0) != IMAP_CMD_OK))
+  if (lastdata && (imap_exec (lastdata, NULL, IMAP_CMD_FAIL_OK) == -1))
   {
-    dprint (1, (debugfile, "Error polling mailboxes"));
+    dprint (1, (debugfile, "Error polling mailboxes\n"));
     return 0;
   }
 
@@ -1892,6 +1892,7 @@ int imap_complete(char* dest, size_t dlen, char* path) {
 
   /* and see what the results are */
   strfcpy (completion, NONULL(mx.mbox), sizeof(completion));
+  idata->cmdtype = IMAP_CT_LIST;
   idata->cmddata = &listresp;
   do
   {
