@@ -64,7 +64,7 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
   char hdrreq[STRING];
   FILE *fp;
   char tempfile[_POSIX_PATH_MAX];
-  int msgno;
+  int msgno, idx;
   IMAP_HEADER h;
   IMAP_STATUS* status;
   int rc, mfhrc, oldmsgcount;
@@ -115,7 +115,7 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
     mx_alloc_memory (idata->ctx);
 
   oldmsgcount = ctx->msgcount;
-  idata->reopen &= ~IMAP_NEWMAIL_PENDING;
+  idata->reopen &= ~(IMAP_REOPEN_ALLOW|IMAP_NEWMAIL_PENDING);
   idata->newMailCount = 0;
 
 #if USE_HCACHE
@@ -166,26 +166,25 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
           break;
 	}
 
+        idx = h.sid - 1;
         sprintf(uid_buf, "/%u", h.data->uid); /* XXX --tg 21:41 04-07-11 */
         uid_validity = (unsigned int*)mutt_hcache_fetch (hc, uid_buf, &imap_hcache_keylen);
 
         if (uid_validity != NULL && *uid_validity == idata->uid_validity)
         {
-  	  ctx->hdrs[msgno] = mutt_hcache_restore((unsigned char *) uid_validity, 0);
-  	  ctx->hdrs[msgno]->index = h.sid - 1;
-  	  if (h.sid != ctx->msgcount + 1)
-  	    dprint (1, (debugfile, "imap_read_headers: msgcount and sequence ID are inconsistent!"));
+  	  ctx->hdrs[idx] = mutt_hcache_restore((unsigned char *) uid_validity, 0);
+  	  ctx->hdrs[idx]->index = idx;
   	  /* messages which have not been expunged are ACTIVE (borrowed from mh 
   	   * folders) */
-  	  ctx->hdrs[msgno]->active = 1;
-          ctx->hdrs[msgno]->read = h.data->read;
-          ctx->hdrs[msgno]->old = h.data->old;
-          ctx->hdrs[msgno]->deleted = h.data->deleted;
-          ctx->hdrs[msgno]->flagged = h.data->flagged;
-          ctx->hdrs[msgno]->replied = h.data->replied;
-          ctx->hdrs[msgno]->changed = h.data->changed;
+  	  ctx->hdrs[idx]->active = 1;
+          ctx->hdrs[idx]->read = h.data->read;
+          ctx->hdrs[idx]->old = h.data->old;
+          ctx->hdrs[idx]->deleted = h.data->deleted;
+          ctx->hdrs[idx]->flagged = h.data->flagged;
+          ctx->hdrs[idx]->replied = h.data->replied;
+          ctx->hdrs[idx]->changed = h.data->changed;
           /*  ctx->hdrs[msgno]->received is restored from mutt_hcache_restore */
-          ctx->hdrs[msgno]->data = (void *) (h.data);
+          ctx->hdrs[idx]->data = (void *) (h.data);
 
           ctx->msgcount++;
         }
@@ -207,19 +206,13 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
         return -1;
       }
     }
-  
-    fetchlast = msgbegin;
+    /* could also look for first null header in case hcache is holey */
+    msgbegin = ctx->msgcount;
   }
 #endif /* USE_HCACHE */
 
   for (msgno = msgbegin; msgno <= msgend ; msgno++)
   {
-    if (ctx->hdrs[msgno])
-    {
-      msgbegin++;
-      continue;
-    }
-
     if (ReadInc && (msgno == msgbegin || ((msgno+1) % ReadInc == 0)))
       mutt_message (_("Fetching message headers... [%d/%d]"), msgno + 1,
         msgend + 1);
@@ -244,7 +237,6 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
       imap_cmd_start (idata, buf);
     }
 
-    /* freshen fp, h */
     rewind (fp);
     memset (&h, 0, sizeof (h));
     h.data = safe_calloc (1, sizeof (IMAP_HEADER_DATA));
@@ -269,23 +261,21 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
       /* make sure we don't get remnants from older larger message headers */
       fputs ("\n\n", fp);
 
-      /* update context with message header */
-      ctx->hdrs[msgno] = mutt_new_header ();
+      idx = h.sid - 1;
+      ctx->hdrs[idx] = mutt_new_header ();
 
-      ctx->hdrs[msgno]->index = h.sid - 1;
-      if (h.sid != ctx->msgcount + 1)
-	dprint (1, (debugfile, "imap_read_headers: msgcount and sequence ID are inconsistent!"));
+      ctx->hdrs[idx]->index = h.sid - 1;
       /* messages which have not been expunged are ACTIVE (borrowed from mh 
        * folders) */
-      ctx->hdrs[msgno]->active = 1;
-      ctx->hdrs[msgno]->read = h.data->read;
-      ctx->hdrs[msgno]->old = h.data->old;
-      ctx->hdrs[msgno]->deleted = h.data->deleted;
-      ctx->hdrs[msgno]->flagged = h.data->flagged;
-      ctx->hdrs[msgno]->replied = h.data->replied;
-      ctx->hdrs[msgno]->changed = h.data->changed;
-      ctx->hdrs[msgno]->received = h.received;
-      ctx->hdrs[msgno]->data = (void *) (h.data);
+      ctx->hdrs[idx]->active = 1;
+      ctx->hdrs[idx]->read = h.data->read;
+      ctx->hdrs[idx]->old = h.data->old;
+      ctx->hdrs[idx]->deleted = h.data->deleted;
+      ctx->hdrs[idx]->flagged = h.data->flagged;
+      ctx->hdrs[idx]->replied = h.data->replied;
+      ctx->hdrs[idx]->changed = h.data->changed;
+      ctx->hdrs[idx]->received = h.received;
+      ctx->hdrs[idx]->data = (void *) (h.data);
 
       if (maxuid < h.data->uid)
         maxuid = h.data->uid;
@@ -293,14 +283,14 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
       rewind (fp);
       /* NOTE: if Date: header is missing, mutt_read_rfc822_header depends
        *   on h.received being set */
-      ctx->hdrs[msgno]->env = mutt_read_rfc822_header (fp, ctx->hdrs[msgno],
+      ctx->hdrs[idx]->env = mutt_read_rfc822_header (fp, ctx->hdrs[idx],
         0, 0);
       /* content built as a side-effect of mutt_read_rfc822_header */
-      ctx->hdrs[msgno]->content->length = h.content_length;
+      ctx->hdrs[idx]->content->length = h.content_length;
 
 #if USE_HCACHE
       sprintf(uid_buf, "/%u", h.data->uid);
-      mutt_hcache_store(hc, uid_buf, ctx->hdrs[msgno], idata->uid_validity, &imap_hcache_keylen);
+      mutt_hcache_store(hc, uid_buf, ctx->hdrs[idx], idata->uid_validity, &imap_hcache_keylen);
 #endif /* USE_HCACHE */
 
       ctx->msgcount++;
@@ -354,6 +344,7 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
     mx_update_context (ctx, ctx->msgcount - oldmsgcount);
   }
 
+  idata->reopen |= IMAP_REOPEN_ALLOW;
   return msgend;
 }
 
