@@ -328,15 +328,17 @@ int imap_get_literal_count(const char *buf, long *bytes)
   char *pc;
   char *pn;
 
-  if (!(pc = strchr (buf, '{')))
-    return (-1);
+  if (!buf || !(pc = strchr (buf, '{')))
+    return -1;
+
   pc++;
   pn = pc;
   while (isdigit ((unsigned char) *pc))
     pc++;
   *pc = 0;
   *bytes = atoi(pn);
-  return (0);
+
+  return 0;
 }
 
 /* imap_get_qualifier: in a tagged response, skip tag and status for
