@@ -13,7 +13,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */ 
 
 #if HAVE_CONFIG_H
@@ -31,7 +31,7 @@
 #include "mutt_crypt.h"
 #include "mutt_idna.h"
 
-#if defined(USE_SSL) || defined(USE_GNUTLS)
+#if defined(USE_SSL)
 #include "mutt_ssl.h"
 #endif
 
@@ -49,6 +49,30 @@
 #include <errno.h>
 #include <sys/wait.h>
 
+#define CHECK_PAGER \
+  if ((CurrentMenu == MENU_PAGER) && (idx >= 0) &&	\
+	    (MuttVars[idx].flags & R_RESORT)) \
+	{ \
+	  snprintf (err->data, err->dsize, \
+	    _("Not available in this menu.")); \
+	  return (-1); \
+	} else
+
+typedef struct myvar
+{
+  char *name;
+  char *value;
+  struct myvar* next;
+} myvar_t;
+
+static myvar_t* MyVars;
+
+static int var_to_string (int idx, char* val, size_t len);
+
+static void myvar_set (const char* var, const char* val);
+static const char* myvar_get (const char* var);
+static void myvar_del (const char* var);
+
 void toggle_quadoption (int opt)
 {
   int n = opt/4;
@@ -123,7 +147,7 @@ int mutt_extract_token (BUFFER *dest, BUFFER *tok, int flags)
 	  (ch == '#' && !(flags & M_TOKEN_COMMENT)) ||
 	  (ch == '=' && (flags & M_TOKEN_EQUAL)) ||
 	  (ch == ';' && !(flags & M_TOKEN_SEMICOLON)) ||
-	  ((flags & M_TOKEN_PATTERN) && strchr ("~=!|", ch)))
+	  ((flags & M_TOKEN_PATTERN) && strchr ("~%=!|", ch)))
 	break;
     }
 
@@ -259,7 +283,9 @@ int mutt_extract_token (BUFFER *dest, BUFFER *tok, int flags)
     }
     else if (ch == '$' && (!qc || qc == '"') && (*tok->dptr == '{' || isalpha ((unsigned char) *tok->dptr)))
     {
-      char *env = NULL, *var = NULL;
+      const char *env = NULL;
+      char *var = NULL;
+      int idx;
 
       if (*tok->dptr == '{')
       {
@@ -277,9 +303,20 @@ int mutt_extract_token (BUFFER *dest, BUFFER *tok, int flags)
 	var = mutt_substrdup (tok->dptr, pc);
 	tok->dptr = pc;
       }
-      if (var && (env = getenv (var)))
-	mutt_buffer_addstr (dest, env);
-      FREE (&var);
+      if (var)
+      {
+        if ((env = getenv (var)) || (env = myvar_get (var)))
+          mutt_buffer_addstr (dest, env);
+        else if ((idx = mutt_option_index (var)) != -1)
+        {
+          /* expand settable mutt variables */
+          char val[LONG_STRING];
+
+          if (var_to_string (idx, val, sizeof (val)))
+            mutt_buffer_addstr (dest, val);
+        }
+        FREE (&var);
+      }
     }
     else
       mutt_buffer_addch (dest, ch);
@@ -309,7 +346,7 @@ static void mutt_free_opt (struct option_t* p)
     break;
   case DT_PATH:
   case DT_STR:
-    FREE ((char**)p->data);
+    FREE ((char**)p->data);		/* __FREE_CHECKED__ */
     break;
   }
 }
@@ -366,7 +403,7 @@ static void add_to_list (LIST **list, const char *str)
   }
 }
 
-static int add_to_rx_list (RX_LIST **list, const char *s, int flags, BUFFER *err)
+int mutt_add_to_rx_list (RX_LIST **list, const char *s, int flags, BUFFER *err)
 {
   RX_LIST *t, *last = NULL;
   REGEXP *rx;
@@ -440,7 +477,7 @@ static int add_to_spam_list (SPAM_LIST **list, const char *pat, const char *temp
        * the template, and leaving t pointed at the current item.
        */
       t = last;
-      safe_free(&t->template);
+      FREE(&t->template);
       break;
     }
     if (!last->next)
@@ -496,8 +533,8 @@ static int remove_from_spam_list (SPAM_LIST **list, const char *pat)
   {
     *list = spam->next;
     mutt_free_regexp(&spam->rx);
-    safe_free(&spam->template);
-    safe_free(&spam);
+    FREE(&spam->template);
+    FREE(&spam);
     return 1;
   }
 
@@ -508,8 +545,8 @@ static int remove_from_spam_list (SPAM_LIST **list, const char *pat)
     {
       prev->next = spam->next;
       mutt_free_regexp(&spam->rx);
-      safe_free(&spam->template);
-      safe_free(&spam);
+      FREE(&spam->template);
+      FREE(&spam);
       spam = prev->next;
       ++nremoved;
     }
@@ -641,18 +678,33 @@ static void _alternates_clean (void)
 
 static int parse_alternates (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
 {
+  group_context_t *gc = NULL;
+  
   _alternates_clean();
+
   do
   {
     mutt_extract_token (buf, s, 0);
+
+    if (parse_group_context (&gc, buf, s, data, err) == -1)
+      goto bail;
+
     remove_from_rx_list (&UnAlternates, buf->data);
 
-    if (add_to_rx_list (&Alternates, buf->data, REG_ICASE, err) != 0)
-      return -1;
+    if (mutt_add_to_rx_list (&Alternates, buf->data, REG_ICASE, err) != 0)
+      goto bail;
+
+    if (mutt_group_context_add_rx (gc, buf->data, REG_ICASE, err) != 0)
+      goto bail;
   }
   while (MoreArgs (s));
-
+  
+  mutt_group_context_destroy (&gc);
   return 0;
+  
+ bail:
+  mutt_group_context_destroy (&gc);
+  return -1;
 }
 
 static int parse_unalternates (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
@@ -664,7 +716,7 @@ static int parse_unalternates (BUFFER *buf, BUFFER *s, unsigned long data, BUFFE
     remove_from_rx_list (&Alternates, buf->data);
 
     if (mutt_strcmp (buf->data, "*") &&
-	add_to_rx_list (&UnAlternates, buf->data, REG_ICASE, err) != 0)
+	mutt_add_to_rx_list (&UnAlternates, buf->data, REG_ICASE, err) != 0)
       return -1;
 
   }
@@ -735,7 +787,7 @@ static int parse_spam_list (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *
       return 0;
 
     /* Otherwise, add it to the nospam list. */
-    if (add_to_rx_list (&NoSpamList, buf->data, REG_ICASE, err) != 0)
+    if (mutt_add_to_rx_list (&NoSpamList, buf->data, REG_ICASE, err) != 0)
       return -1;
 
     return 0;
@@ -769,17 +821,360 @@ static int parse_unlist (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err
 
 static int parse_lists (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
 {
+  group_context_t *gc = NULL;
+
   do
   {
     mutt_extract_token (buf, s, 0);
+    
+    if (parse_group_context (&gc, buf, s, data, err) == -1)
+      goto bail;
+    
     remove_from_rx_list (&UnMailLists, buf->data);
     
-    if (add_to_rx_list (&MailLists, buf->data, REG_ICASE, err) != 0)
-      return -1;
+    if (mutt_add_to_rx_list (&MailLists, buf->data, REG_ICASE, err) != 0)
+      goto bail;
+    
+    if (mutt_group_context_add_rx (gc, buf->data, REG_ICASE, err) != 0)
+      goto bail;
   }
   while (MoreArgs (s));
 
+  mutt_group_context_destroy (&gc);
   return 0;
+  
+ bail:
+  mutt_group_context_destroy (&gc);
+  return -1;
+}
+
+typedef enum group_state_t {
+  NONE, RX, ADDR
+} group_state_t;
+
+static int parse_group (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
+{
+  group_context_t *gc = NULL;
+  group_state_t state = NONE;
+  ADDRESS *addr = NULL;
+  char *estr = NULL;
+  
+  do 
+  {
+    mutt_extract_token (buf, s, 0);
+    if (parse_group_context (&gc, buf, s, data, err) == -1)
+      goto bail;
+    
+    if (!mutt_strcasecmp (buf->data, "-rx"))
+      state = RX;
+    else if (!mutt_strcasecmp (buf->data, "-addr"))
+      state = ADDR;
+    else 
+    {
+      switch (state) 
+      {
+	case NONE:
+	  strfcpy (err->data, _("Missing -rx or -addr."), err->dsize);
+	  goto bail;
+	
+	case RX:
+	  if (mutt_group_context_add_rx (gc, buf->data, REG_ICASE, err) != 0)
+	    goto bail;
+	  break;
+	
+	case ADDR:
+	  if ((addr = mutt_parse_adrlist (NULL, buf->data)) == NULL)
+	    goto bail;
+	  if (mutt_addrlist_to_idna (addr, &estr)) 
+	  {
+	    snprintf (err->data, err->dsize, _("Warning: Bad IDN '%s'.\n"),
+		      estr);
+	    goto bail;
+	  }
+	  mutt_group_context_add_adrlist (gc, addr);
+	  rfc822_free_address (&addr);
+	  break;
+      }
+    }
+  } while (MoreArgs (s));
+
+  mutt_group_context_destroy (&gc);
+  return 0;
+
+  bail:
+  mutt_group_context_destroy (&gc);
+  return -1;
+}
+
+static int parse_ungroup (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
+{
+  strfcpy (err->data, "not implemented", err->dsize);
+  return -1;
+}
+
+/* always wise to do what someone else did before */
+static void _attachments_clean (void)
+{
+  int i;
+  if (Context && Context->msgcount) 
+  {
+    for (i = 0; i < Context->msgcount; i++)
+      Context->hdrs[i]->attach_valid = 0;
+  }
+}
+
+static int parse_attach_list (BUFFER *buf, BUFFER *s, LIST **ldata, BUFFER *err)
+{
+  ATTACH_MATCH *a;
+  LIST *listp, *lastp;
+  char *p;
+  char *tmpminor;
+  int len;
+
+  /* Find the last item in the list that data points to. */
+  lastp = NULL;
+  dprint(5, (debugfile, "parse_attach_list: ldata = %08x, *ldata = %08x\n",
+	      (unsigned int)ldata, (unsigned int)*ldata));
+  for (listp = *ldata; listp; listp = listp->next)
+  {
+    a = (ATTACH_MATCH *)listp->data;
+    dprint(5, (debugfile, "parse_attach_list: skipping %s/%s\n",
+		a->major, a->minor));
+    lastp = listp;
+  }
+
+  do
+  {
+    mutt_extract_token (buf, s, 0);
+
+    if (!buf->data || *buf->data == '\0')
+      continue;
+   
+    a = safe_malloc(sizeof(ATTACH_MATCH));
+
+    /* some cheap hacks that I expect to remove */
+    if (!ascii_strcasecmp(buf->data, "any"))
+      a->major = safe_strdup("*/.*");
+    else if (!ascii_strcasecmp(buf->data, "none"))
+      a->major = safe_strdup("cheap_hack/this_should_never_match");
+    else
+      a->major = safe_strdup(buf->data);
+
+    if ((p = strchr(a->major, '/')))
+    {
+      *p = '\0';
+      ++p;
+      a->minor = p;
+    }
+    else
+    {
+      a->minor = "unknown";
+    }
+
+    len = strlen(a->minor);
+    tmpminor = safe_malloc(len+3);
+    strcpy(&tmpminor[1], a->minor); /* __STRCPY_CHECKED__ */
+    tmpminor[0] = '^';
+    tmpminor[len+1] = '$';
+    tmpminor[len+2] = '\0';
+
+    a->major_int = mutt_check_mime_type(a->major);
+    regcomp(&a->minor_rx, tmpminor, REG_ICASE|REG_EXTENDED);
+
+    FREE(&tmpminor);
+
+    dprint(5, (debugfile, "parse_attach_list: added %s/%s [%d]\n",
+		a->major, a->minor, a->major_int));
+
+    listp = safe_malloc(sizeof(LIST));
+    listp->data = (char *)a;
+    listp->next = NULL;
+    if (lastp)
+    {
+      lastp->next = listp;
+    }
+    else
+    {
+      *ldata = listp;
+    }
+    lastp = listp;
+  }
+  while (MoreArgs (s));
+   
+  _attachments_clean();
+  return 0;
+}
+
+static int parse_unattach_list (BUFFER *buf, BUFFER *s, LIST **ldata, BUFFER *err)
+{
+  ATTACH_MATCH *a;
+  LIST *lp, *lastp, *newlp;
+  char *tmp;
+  int major;
+  char *minor;
+
+  do
+  {
+    mutt_extract_token (buf, s, 0);
+
+    if (!ascii_strcasecmp(buf->data, "any"))
+      tmp = safe_strdup("*/.*");
+    else if (!ascii_strcasecmp(buf->data, "none"))
+      tmp = safe_strdup("cheap_hack/this_should_never_match");
+    else
+      tmp = safe_strdup(buf->data);
+
+    if ((minor = strchr(tmp, '/')))
+    {
+      *minor = '\0';
+      ++minor;
+    }
+    else
+    {
+      minor = "unknown";
+    }
+    major = mutt_check_mime_type(tmp);
+
+    /* We must do our own walk here because remove_from_list() will only
+     * remove the LIST->data, not anything pointed to by the LIST->data. */
+    lastp = NULL;
+    for(lp = *ldata; lp; )
+    {
+      a = (ATTACH_MATCH *)lp->data;
+      dprint(5, (debugfile, "parse_unattach_list: check %s/%s [%d] : %s/%s [%d]\n",
+		  a->major, a->minor, a->major_int, tmp, minor, major));
+      if (a->major_int == major && !mutt_strcasecmp(minor, a->minor))
+      {
+	dprint(5, (debugfile, "parse_unattach_list: removed %s/%s [%d]\n",
+		    a->major, a->minor, a->major_int));
+	regfree(&a->minor_rx);
+	FREE(&a->major);
+
+	/* Relink backward */
+	if (lastp)
+	  lastp->next = lp->next;
+	else
+	  *ldata = lp->next;
+
+        newlp = lp->next;
+        FREE(&lp->data);	/* same as a */
+        FREE(&lp);
+        lp = newlp;
+        continue;
+      }
+
+      lastp = lp;
+      lp = lp->next;
+    }
+
+  }
+  while (MoreArgs (s));
+   
+  FREE(&tmp);
+  _attachments_clean();
+  return 0;
+}
+
+static int print_attach_list (LIST *lp, char op, char *name)
+{
+  while (lp) {
+    printf("attachments %c%s %s/%s\n", op, name,
+           ((ATTACH_MATCH *)lp->data)->major,
+           ((ATTACH_MATCH *)lp->data)->minor);
+    lp = lp->next;
+  }
+
+  return 0;
+}
+
+
+static int parse_attachments (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
+{
+  char op, *category;
+  LIST **listp;
+
+  mutt_extract_token(buf, s, 0);
+  if (!buf->data || *buf->data == '\0') {
+    strfcpy(err->data, _("attachments: no disposition"), err->dsize);
+    return -1;
+  }
+
+  category = buf->data;
+  op = *category++;
+
+  if (op == '?') {
+    mutt_endwin (NULL);
+    fflush (stdout);
+    printf("\nCurrent attachments settings:\n\n");
+    print_attach_list(AttachAllow,   '+', "A");
+    print_attach_list(AttachExclude, '-', "A");
+    print_attach_list(InlineAllow,   '+', "I");
+    print_attach_list(InlineExclude, '-', "I");
+    set_option (OPTFORCEREDRAWINDEX);
+    set_option (OPTFORCEREDRAWPAGER);
+    mutt_any_key_to_continue (NULL);
+    return 0;
+  }
+
+  if (op != '+' && op != '-') {
+    op = '+';
+    category--;
+  }
+  if (!ascii_strncasecmp(category, "attachment", strlen(category))) {
+    if (op == '+')
+      listp = &AttachAllow;
+    else
+      listp = &AttachExclude;
+  }
+  else if (!ascii_strncasecmp(category, "inline", strlen(category))) {
+    if (op == '+')
+      listp = &InlineAllow;
+    else
+      listp = &InlineExclude;
+  }
+  else {
+    strfcpy(err->data, _("attachments: invalid disposition"), err->dsize);
+    return -1;
+  }
+
+  return parse_attach_list(buf, s, listp, err);
+}
+
+static int parse_unattachments (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
+{
+  char op, *p;
+  LIST **listp;
+
+  mutt_extract_token(buf, s, 0);
+  if (!buf->data || *buf->data == '\0') {
+    strfcpy(err->data, _("unattachments: no disposition"), err->dsize);
+    return -1;
+  }
+
+  p = buf->data;
+  op = *p++;
+  if (op != '+' && op != '-') {
+    op = '+';
+    p--;
+  }
+  if (!ascii_strncasecmp(p, "attachment", strlen(p))) {
+    if (op == '+')
+      listp = &AttachAllow;
+    else
+      listp = &AttachExclude;
+  }
+  else if (!ascii_strncasecmp(p, "inline", strlen(p))) {
+    if (op == '+')
+      listp = &InlineAllow;
+    else
+      listp = &InlineExclude;
+  }
+  else {
+    strfcpy(err->data, _("unattachments: invalid disposition"), err->dsize);
+    return -1;
+  }
+
+  return parse_unattach_list(buf, s, listp, err);
 }
 
 static int parse_unlists (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
@@ -791,7 +1186,7 @@ static int parse_unlists (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *er
     remove_from_rx_list (&MailLists, buf->data);
     
     if (mutt_strcmp (buf->data, "*") && 
-	add_to_rx_list (&UnMailLists, buf->data, REG_ICASE, err) != 0)
+	mutt_add_to_rx_list (&UnMailLists, buf->data, REG_ICASE, err) != 0)
       return -1;
   }
   while (MoreArgs (s));
@@ -801,20 +1196,33 @@ static int parse_unlists (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *er
 
 static int parse_subscribe (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
 {
+  group_context_t *gc = NULL;
+  
   do
   {
     mutt_extract_token (buf, s, 0);
+
+    if (parse_group_context (&gc, buf, s, data, err) == -1)
+      goto bail;
+    
     remove_from_rx_list (&UnMailLists, buf->data);
     remove_from_rx_list (&UnSubscribedLists, buf->data);
 
-    if (add_to_rx_list (&MailLists, buf->data, REG_ICASE, err) != 0)
-      return -1;
-    if (add_to_rx_list (&SubscribedLists, buf->data, REG_ICASE, err) != 0)
-      return -1;
+    if (mutt_add_to_rx_list (&MailLists, buf->data, REG_ICASE, err) != 0)
+      goto bail;
+    if (mutt_add_to_rx_list (&SubscribedLists, buf->data, REG_ICASE, err) != 0)
+      goto bail;
+    if (mutt_group_context_add_rx (gc, buf->data, REG_ICASE, err) != 0)
+      goto bail;
   }
   while (MoreArgs (s));
-
+  
+  mutt_group_context_destroy (&gc);
   return 0;
+  
+ bail:
+  mutt_group_context_destroy (&gc);
+  return -1;
 }
 
 static int parse_unsubscribe (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
@@ -825,7 +1233,7 @@ static int parse_unsubscribe (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER
     remove_from_rx_list (&SubscribedLists, buf->data);
     
     if (mutt_strcmp (buf->data, "*") &&
-	add_to_rx_list (&UnSubscribedLists, buf->data, REG_ICASE, err) != 0)
+	mutt_add_to_rx_list (&UnSubscribedLists, buf->data, REG_ICASE, err) != 0)
       return -1;
   }
   while (MoreArgs (s));
@@ -885,6 +1293,7 @@ static int parse_alias (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
   ALIAS *tmp = Aliases;
   ALIAS *last = NULL;
   char *estr = NULL;
+  group_context_t *gc = NULL;
   
   if (!MoreArgs (s))
   {
@@ -894,8 +1303,8 @@ static int parse_alias (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
 
   mutt_extract_token (buf, s, 0);
 
-  dprint (2, (debugfile, "parse_alias: First token is '%s'.\n",
-	      buf->data));
+  if (parse_group_context (&gc, buf, s, data, err) == -1)
+    return -1;
   
   /* check to see if an alias with this name already exists */
   for (; tmp; tmp = tmp->next)
@@ -926,7 +1335,9 @@ static int parse_alias (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
   mutt_extract_token (buf, s, M_TOKEN_QUOTE | M_TOKEN_SPACE | M_TOKEN_SEMICOLON);
   dprint (2, (debugfile, "parse_alias: Second token is '%s'.\n",
 	      buf->data));
+
   tmp->addr = mutt_parse_adrlist (tmp->addr, buf->data);
+
   if (last)
     last->next = tmp;
   else
@@ -935,13 +1346,18 @@ static int parse_alias (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
   {
     snprintf (err->data, err->dsize, _("Warning: Bad IDN '%s' in alias '%s'.\n"),
 	      estr, tmp->name);
-    return -1;
+    goto bail;
   }
+
+  mutt_group_context_add_adrlist (gc, tmp->addr);
+
+
 #ifdef DEBUG
   if (debuglevel >= 2) 
   {
     ADDRESS *a;
-    for (a = tmp->addr; a; a = a->next)
+    /* A group is terminated with an empty address, so check a->mailbox */
+    for (a = tmp->addr; a && a->mailbox; a = a->next)
     {
       if (!a->group)
 	dprint (2, (debugfile, "parse_alias:   %s\n",
@@ -952,7 +1368,12 @@ static int parse_alias (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
     }
   }
 #endif
+  mutt_group_context_destroy (&gc);
   return 0;
+  
+  bail:
+  mutt_group_context_destroy (&gc);
+  return -1;
 }
 
 static int
@@ -1201,8 +1622,10 @@ static void mutt_restore_default (struct option_t *p)
 
 static int parse_set (BUFFER *tmp, BUFFER *s, unsigned long data, BUFFER *err)
 {
-  int idx, query, unset, inv, reset, r = 0;
+  int query, unset, inv, reset, r = 0;
+  int idx = -1;
   char *p, scratch[_POSIX_PATH_MAX];
+  char* myvar;
 
   while (MoreArgs (s))
   {
@@ -1211,6 +1634,7 @@ static int parse_set (BUFFER *tmp, BUFFER *s, unsigned long data, BUFFER *err)
     unset = data & M_SET_UNSET;
     inv = data & M_SET_INV;
     reset = data & M_SET_RESET;
+    myvar = NULL;
 
     if (*s->dptr == '?')
     {
@@ -1236,7 +1660,9 @@ static int parse_set (BUFFER *tmp, BUFFER *s, unsigned long data, BUFFER *err)
     /* get the variable name */
     mutt_extract_token (tmp, s, M_TOKEN_EQUAL);
 
-    if ((idx = mutt_option_index (tmp->data)) == -1 &&
+    if (!mutt_strncmp ("my_", tmp->data, 3))
+      myvar = tmp->data;
+    else if ((idx = mutt_option_index (tmp->data)) == -1 &&
 	!(reset && !mutt_strcmp ("all", tmp->data)))
     {
       snprintf (err->data, err->dsize, _("%s: unknown variable"), tmp->data);
@@ -1260,14 +1686,31 @@ static int parse_set (BUFFER *tmp, BUFFER *s, unsigned long data, BUFFER *err)
      
       if (!mutt_strcmp ("all", tmp->data))
       {
+	if (CurrentMenu == MENU_PAGER)
+	{
+	  snprintf (err->data, err->dsize, _("Not available in this menu."));
+	  return (-1);
+	}
 	for (idx = 0; MuttVars[idx].option; idx++)
 	  mutt_restore_default (&MuttVars[idx]);
+	set_option (OPTFORCEREDRAWINDEX);
+	set_option (OPTFORCEREDRAWPAGER);
+	set_option (OPTSORTSUBTHREADS);
+	set_option (OPTNEEDRESORT);
+	set_option (OPTRESORTINIT);
+	set_option (OPTREDRAWTREE);
 	return 0;
       }
       else
-	mutt_restore_default (&MuttVars[idx]);
+      {
+	CHECK_PAGER;
+        if (myvar)
+          myvar_del (myvar);
+        else
+          mutt_restore_default (&MuttVars[idx]);
+      }
     } 
-    else if (DTYPE (MuttVars[idx].type) == DT_BOOL)
+    else if (!myvar && DTYPE (MuttVars[idx].type) == DT_BOOL)
     { 
       if (s && *s->dptr == '=')
       {
@@ -1297,6 +1740,7 @@ static int parse_set (BUFFER *tmp, BUFFER *s, unsigned long data, BUFFER *err)
 	return 0;
       }
 
+      CHECK_PAGER;
       if (unset)
 	unset_option (MuttVars[idx].data);
       else if (inv)
@@ -1304,23 +1748,41 @@ static int parse_set (BUFFER *tmp, BUFFER *s, unsigned long data, BUFFER *err)
       else
 	set_option (MuttVars[idx].data);
     }
-    else if (DTYPE (MuttVars[idx].type) == DT_STR ||
+    else if (myvar || DTYPE (MuttVars[idx].type) == DT_STR ||
 	     DTYPE (MuttVars[idx].type) == DT_PATH ||
 	     DTYPE (MuttVars[idx].type) == DT_ADDR)
     {
       if (unset)
       {
-	if (DTYPE (MuttVars[idx].type) == DT_ADDR)
+	CHECK_PAGER;
+        if (myvar)
+          myvar_del (myvar);
+	else if (DTYPE (MuttVars[idx].type) == DT_ADDR)
 	  rfc822_free_address ((ADDRESS **) MuttVars[idx].data);
 	else
-	  FREE ((void *) MuttVars[idx].data);
+	  /* MuttVars[idx].data is already 'char**' (or some 'void**') or... 
+	   * so cast to 'void*' is okay */
+	  FREE ((void *) MuttVars[idx].data);		/* __FREE_CHECKED__ */
       }
       else if (query || *s->dptr != '=')
       {
 	char _tmp[STRING];
-	char *val = NULL;
-	
-	if (DTYPE (MuttVars[idx].type) == DT_ADDR)
+	const char *val = NULL;
+
+        if (myvar)
+        {
+          if ((val = myvar_get (myvar)))
+          {
+            snprintf (err->data, err->dsize, "%s=\"%s\"", myvar, val);
+            break;
+          }
+          else
+          {
+            snprintf (err->data, err->dsize, _("%s: unknown variable"), myvar);
+            return (-1);
+          }
+        }
+	else if (DTYPE (MuttVars[idx].type) == DT_ADDR)
 	{
 	  _tmp[0] = '\0';
 	  rfc822_write_address (_tmp, sizeof (_tmp), *((ADDRESS **) MuttVars[idx].data), 0);
@@ -1336,16 +1798,30 @@ static int parse_set (BUFFER *tmp, BUFFER *s, unsigned long data, BUFFER *err)
       }
       else
       {
+	CHECK_PAGER;
         s->dptr++;
 
-        /* copy the value of the string */
-        if (DTYPE (MuttVars[idx].type) == DT_ADDR)
+        if (myvar)
+	{
+	  /* myvar is a pointer to tmp and will be lost on extract_token */
+	  myvar = safe_strdup (myvar);
+          myvar_del (myvar);
+	}
+        else if (DTYPE (MuttVars[idx].type) == DT_ADDR)
 	  rfc822_free_address ((ADDRESS **) MuttVars[idx].data);
         else
-	  FREE ((void *) MuttVars[idx].data);
+	  /* MuttVars[idx].data is already 'char**' (or some 'void**') or... 
+	   * so cast to 'void*' is okay */
+	  FREE ((void *) MuttVars[idx].data);		/* __FREE_CHECKED__ */
 
         mutt_extract_token (tmp, s, 0);
-        if (DTYPE (MuttVars[idx].type) == DT_PATH)
+        if (myvar)
+        {
+          myvar_set (myvar, tmp->data);
+          FREE (&myvar);
+	  myvar="don't resort";
+        }
+        else if (DTYPE (MuttVars[idx].type) == DT_PATH)
         {
 	  strfcpy (scratch, tmp->data, sizeof (scratch));
 	  mutt_expand_path (scratch, sizeof (scratch));
@@ -1384,6 +1860,7 @@ static int parse_set (BUFFER *tmp, BUFFER *s, unsigned long data, BUFFER *err)
 	break;
       }
       
+      CHECK_PAGER;
       s->dptr++;
 
       /* copy the value of the string */
@@ -1477,6 +1954,7 @@ static int parse_set (BUFFER *tmp, BUFFER *s, unsigned long data, BUFFER *err)
 	break;
       }
 
+      CHECK_PAGER;
       s->dptr++;
 
       /* copy the value of the string */
@@ -1501,6 +1979,7 @@ static int parse_set (BUFFER *tmp, BUFFER *s, unsigned long data, BUFFER *err)
 	break;
       }
 
+      CHECK_PAGER;
       s->dptr++;
 
       mutt_extract_token (tmp, s, 0);
@@ -1539,6 +2018,7 @@ static int parse_set (BUFFER *tmp, BUFFER *s, unsigned long data, BUFFER *err)
 	break;
       }
 
+      CHECK_PAGER;
       if (*s->dptr == '=')
       {
 	s->dptr++;
@@ -1609,6 +2089,7 @@ static int parse_set (BUFFER *tmp, BUFFER *s, unsigned long data, BUFFER *err)
 		  p);
 	return 0;
       }
+      CHECK_PAGER;
       s->dptr++;
       mutt_extract_token (tmp, s , 0);
 
@@ -1625,18 +2106,21 @@ static int parse_set (BUFFER *tmp, BUFFER *s, unsigned long data, BUFFER *err)
       break;
     }
 
-    if (MuttVars[idx].flags & R_INDEX)
-      set_option (OPTFORCEREDRAWINDEX);
-    if (MuttVars[idx].flags & R_PAGER)
-      set_option (OPTFORCEREDRAWPAGER);
-    if (MuttVars[idx].flags & R_RESORT_SUB)
-      set_option (OPTSORTSUBTHREADS);
-    if (MuttVars[idx].flags & R_RESORT)
-      set_option (OPTNEEDRESORT);
-    if (MuttVars[idx].flags & R_RESORT_INIT)
-      set_option (OPTRESORTINIT);
-    if (MuttVars[idx].flags & R_TREE)
-      set_option (OPTREDRAWTREE);
+    if (!myvar)
+    {
+      if (MuttVars[idx].flags & R_INDEX)
+        set_option (OPTFORCEREDRAWINDEX);
+      if (MuttVars[idx].flags & R_PAGER)
+        set_option (OPTFORCEREDRAWPAGER);
+      if (MuttVars[idx].flags & R_RESORT_SUB)
+        set_option (OPTSORTSUBTHREADS);
+      if (MuttVars[idx].flags & R_RESORT)
+        set_option (OPTNEEDRESORT);
+      if (MuttVars[idx].flags & R_RESORT_INIT)
+        set_option (OPTRESORTINIT);
+      if (MuttVars[idx].flags & R_TREE)
+        set_option (OPTREDRAWTREE);
+    }
   }
   return (r);
 }
@@ -1793,7 +2277,25 @@ char User_typed [LONG_STRING] = {0};
 
 int  Num_matched = 0; /* Number of matches for completion */
 char Completed [STRING] = {0}; /* completed string (command or variable) */
-char *Matches[MAX(NUMVARS,NUMCOMMANDS) + 1]; /* all the matches + User_typed */
+const char **Matches;
+/* this is a lie until mutt_init runs: */
+int  Matches_listsize = MAX(NUMVARS,NUMCOMMANDS) + 10;
+
+static void matches_ensure_morespace(int current)
+{
+  int base_space, extra_space, space;
+
+  if (current > Matches_listsize - 2)
+  {
+    base_space = MAX(NUMVARS,NUMCOMMANDS) + 1; 
+    extra_space = Matches_listsize - base_space;
+    extra_space *= 2;
+    space = base_space + extra_space;
+    safe_realloc (&Matches, space * sizeof (char *));
+    memset (&Matches[current + 1], 0, space - current);
+    Matches_listsize = space;
+  }
+}
 
 /* helper function for completion.  Changes the dest buffer if
    necessary/possible to aid completion.
@@ -1802,12 +2304,13 @@ char *Matches[MAX(NUMVARS,NUMCOMMANDS) + 1]; /* all the matches + User_typed */
 	try == user entered data for completion.
 	len == length of dest buffer.
 */
-static void candidate (char *dest, char *try, char *src, int len)
+static void candidate (char *dest, char *try, const char *src, int len)
 {
   int l;
 
   if (strstr (src, try) == src)
   {
+    matches_ensure_morespace (Num_matched);
     Matches[Num_matched++] = src;
     if (dest[0] == 0)
       strfcpy (dest, src, len);
@@ -1824,6 +2327,7 @@ int mutt_command_complete (char *buffer, size_t len, int pos, int numtabs)
   char *pt = buffer;
   int num;
   int spaces; /* keep track of the number of leading spaces on the line */
+  myvar_t *myv;
 
   SKIPWS (buffer);
   spaces = buffer - pt;
@@ -1839,10 +2343,11 @@ int mutt_command_complete (char *buffer, size_t len, int pos, int numtabs)
     {
       Num_matched = 0;
       strfcpy (User_typed, pt, sizeof (User_typed));
-      memset (Matches, 0, sizeof (Matches));
+      memset (Matches, 0, Matches_listsize);
       memset (Completed, 0, sizeof (Completed));
       for (num = 0; Commands[num].name; num++)
 	candidate (Completed, User_typed, Commands[num].name, sizeof (Completed));
+      matches_ensure_morespace (Num_matched);
       Matches[Num_matched++] = User_typed;
 
       /* All matches are stored. Longest non-ambiguous string is ""
@@ -1892,10 +2397,13 @@ int mutt_command_complete (char *buffer, size_t len, int pos, int numtabs)
     {
       Num_matched = 0;
       strfcpy (User_typed, pt, sizeof (User_typed));
-      memset (Matches, 0, sizeof (Matches));
+      memset (Matches, 0, Matches_listsize);
       memset (Completed, 0, sizeof (Completed));
       for (num = 0; MuttVars[num].option; num++)
 	candidate (Completed, User_typed, MuttVars[num].option, sizeof (Completed));
+      for (myv = MyVars; myv; myv = myv->next)
+	candidate (Completed, User_typed, myv->name, sizeof (Completed));
+      matches_ensure_morespace (Num_matched);
       Matches[Num_matched++] = User_typed;
 
       /* All matches are stored. Longest non-ambiguous string is ""
@@ -1931,7 +2439,7 @@ int mutt_command_complete (char *buffer, size_t len, int pos, int numtabs)
     {
       Num_matched = 0;
       strfcpy (User_typed, pt, sizeof (User_typed));
-      memset (Matches, 0, sizeof (Matches));
+      memset (Matches, 0, Matches_listsize);
       memset (Completed, 0, sizeof (Completed));
       for (num = 0; menu[num].name; num++)
 	candidate (Completed, User_typed, menu[num].name, sizeof (Completed));
@@ -1942,6 +2450,7 @@ int mutt_command_complete (char *buffer, size_t len, int pos, int numtabs)
 	for (num = 0; menu[num].name; num++)
 	  candidate (Completed, User_typed, menu[num].name, sizeof (Completed));
       }
+      matches_ensure_morespace (Num_matched);
       Matches[Num_matched++] = User_typed;
 
       /* All matches are stored. Longest non-ambiguous string is ""
@@ -1991,110 +2500,122 @@ int mutt_var_value_complete (char *buffer, size_t len, int pos)
   if (mutt_strncmp (buffer, "set", 3) == 0)
   {
     int idx;
+    char val[LONG_STRING];
+    const char *myvarval;
+
     strfcpy (var, pt, sizeof (var));
     /* ignore the trailing '=' when comparing */
     var[mutt_strlen (var) - 1] = 0;
-    if ((idx = mutt_option_index (var)) == -1) 
-      return 0; /* no such variable. */
-    else
+    if ((idx = mutt_option_index (var)) == -1)
     {
-      char tmp [LONG_STRING], tmp2[LONG_STRING];
-      char *s, *d;
-      size_t dlen = buffer + len - pt - spaces;
-      char *vals[] = { "no", "yes", "ask-no", "ask-yes" };
-
-      tmp[0] = '\0';
-
-      if ((DTYPE(MuttVars[idx].type) == DT_STR) || 
-	  (DTYPE(MuttVars[idx].type) == DT_PATH) ||
-	  (DTYPE(MuttVars[idx].type) == DT_RX))
+      if ((myvarval = myvar_get(var)) != NULL)
       {
-	strfcpy (tmp, NONULL (*((char **) MuttVars[idx].data)), sizeof (tmp));
-	if (DTYPE (MuttVars[idx].type) == DT_PATH)
-	  mutt_pretty_mailbox (tmp);
+	snprintf (pt, len - (pt - buffer), "%s=\"%s\"", var, myvarval);
+	return 1;
       }
-      else if (DTYPE (MuttVars[idx].type) == DT_ADDR)
-      {
-	rfc822_write_address (tmp, sizeof (tmp), *((ADDRESS **) MuttVars[idx].data), 0);
-      }
-      else if (DTYPE (MuttVars[idx].type) == DT_QUAD)
-	strfcpy (tmp, vals[quadoption (MuttVars[idx].data)], sizeof (tmp));
-      else if (DTYPE (MuttVars[idx].type) == DT_NUM)
-	snprintf (tmp, sizeof (tmp), "%d", (*((short *) MuttVars[idx].data)));
-      else if (DTYPE (MuttVars[idx].type) == DT_SORT)
-      {
-	const struct mapping_t *map;
-	char *p;
-
-	switch (MuttVars[idx].type & DT_SUBTYPE_MASK)
-	{
-	  case DT_SORT_ALIAS:
-	    map = SortAliasMethods;
-	    break;
-	  case DT_SORT_BROWSER:
-	    map = SortBrowserMethods;
-	    break;
-	  case DT_SORT_KEYS:
-            if ((WithCrypto & APPLICATION_PGP))
-              map = SortKeyMethods;
-            else
-              map = SortMethods;
-	    break;
-	  default:
-	    map = SortMethods;
-	    break;
-	}
-	p = mutt_getnamebyvalue (*((short *) MuttVars[idx].data) & SORT_MASK, map);
-	snprintf (tmp, sizeof (tmp), "%s%s%s",
-		  (*((short *) MuttVars[idx].data) & SORT_REVERSE) ? "reverse-" : "",
-		  (*((short *) MuttVars[idx].data) & SORT_LAST) ? "last-" : "",
-		  p);
-      }
-      else if (DTYPE (MuttVars[idx].type) == DT_MAGIC)
-      {
-        char *p;
-
-        switch (DefaultMagic)
-        {
-          case M_MBOX:
-            p = "mbox";
-            break;
-          case M_MMDF:
-            p = "MMDF";
-            break;
-          case M_MH:
-            p = "MH";
-            break;
-          case M_MAILDIR:
-            p = "Maildir";
-            break;
-          default:
-            p = "unknown";
-        }
-        strfcpy (tmp, p, sizeof (tmp));
-      }
-      else if (DTYPE (MuttVars[idx].type) == DT_BOOL)
-	strfcpy (tmp, option (MuttVars[idx].data) ? "yes" : "no", sizeof (tmp));
-      else
-	return 0;
-      
-      for (s = tmp, d = tmp2; *s && (d - tmp2) < sizeof (tmp2) - 2;)
-      {
-	if (*s == '\\' || *s == '"')
-	  *d++ = '\\';
-	*d++ = *s++;
-      }
-      *d = '\0';
-      
-      strfcpy (tmp, pt, sizeof (tmp));
-      snprintf (pt, dlen, "%s\"%s\"", tmp, tmp2);
-	  
+      return 0; /* no such variable. */
+    }
+    else if (var_to_string (idx, val, sizeof (val)))
+    {
+      snprintf (pt, len - (pt - buffer), "%s=\"%s\"", var, val);
       return 1;
     }
   }
   return 0;
 }
 
+static int var_to_string (int idx, char* val, size_t len)
+{
+  char tmp[LONG_STRING];
+  char *s, *d;
+  char *vals[] = { "no", "yes", "ask-no", "ask-yes" };
+
+  tmp[0] = '\0';
+
+  if ((DTYPE(MuttVars[idx].type) == DT_STR) ||
+      (DTYPE(MuttVars[idx].type) == DT_PATH) ||
+      (DTYPE(MuttVars[idx].type) == DT_RX))
+  {
+    strfcpy (tmp, NONULL (*((char **) MuttVars[idx].data)), sizeof (tmp));
+    if (DTYPE (MuttVars[idx].type) == DT_PATH)
+      mutt_pretty_mailbox (tmp);
+  }
+  else if (DTYPE (MuttVars[idx].type) == DT_ADDR)
+  {
+    rfc822_write_address (tmp, sizeof (tmp), *((ADDRESS **) MuttVars[idx].data), 0);
+  }
+  else if (DTYPE (MuttVars[idx].type) == DT_QUAD)
+    strfcpy (tmp, vals[quadoption (MuttVars[idx].data)], sizeof (tmp));
+  else if (DTYPE (MuttVars[idx].type) == DT_NUM)
+    snprintf (tmp, sizeof (tmp), "%d", (*((short *) MuttVars[idx].data)));
+  else if (DTYPE (MuttVars[idx].type) == DT_SORT)
+  {
+    const struct mapping_t *map;
+    char *p;
+
+    switch (MuttVars[idx].type & DT_SUBTYPE_MASK)
+    {
+      case DT_SORT_ALIAS:
+        map = SortAliasMethods;
+        break;
+      case DT_SORT_BROWSER:
+        map = SortBrowserMethods;
+        break;
+      case DT_SORT_KEYS:
+        if ((WithCrypto & APPLICATION_PGP))
+          map = SortKeyMethods;
+        else
+          map = SortMethods;
+        break;
+      default:
+        map = SortMethods;
+        break;
+    }
+    p = mutt_getnamebyvalue (*((short *) MuttVars[idx].data) & SORT_MASK, map);
+    snprintf (tmp, sizeof (tmp), "%s%s%s",
+              (*((short *) MuttVars[idx].data) & SORT_REVERSE) ? "reverse-" : "",
+              (*((short *) MuttVars[idx].data) & SORT_LAST) ? "last-" : "",
+              p);
+  }
+  else if (DTYPE (MuttVars[idx].type) == DT_MAGIC)
+  {
+    char *p;
+
+    switch (DefaultMagic)
+    {
+      case M_MBOX:
+        p = "mbox";
+        break;
+      case M_MMDF:
+        p = "MMDF";
+        break;
+      case M_MH:
+        p = "MH";
+        break;
+      case M_MAILDIR:
+        p = "Maildir";
+        break;
+      default:
+        p = "unknown";
+    }
+    strfcpy (tmp, p, sizeof (tmp));
+  }
+  else if (DTYPE (MuttVars[idx].type) == DT_BOOL)
+    strfcpy (tmp, option (MuttVars[idx].data) ? "yes" : "no", sizeof (tmp));
+  else
+    return 0;
+
+  for (s = tmp, d = val; *s && len - (d - val) > 2; len--)
+  {
+    if (*s == '\\' || *s == '"')
+      *d++ = '\\';
+    *d++ = *s++;
+  }
+  *d = '\0';
+
+  return 1;
+}
+
 /* Implement the -Q command line flag */
 int mutt_query_variables (LIST *queries)
 {
@@ -2145,6 +2666,9 @@ int mutt_dump_variables (void)
   
   for (i = 0; MuttVars[i].option; i++)
   {
+    if (MuttVars[i].type == DT_SYN)
+      continue;
+
     snprintf (command, sizeof (command), "set ?%s\n", MuttVars[i].option);
     if (mutt_parse_rc_line (command, &token, &err) == -1)
     {
@@ -2238,6 +2762,8 @@ void mutt_init (int skip_sys_rc, LIST *commands)
   err.data = error;
   err.dsize = sizeof (error);
 
+  Groups = hash_create (1031);
+  
   /* 
    * XXX - use something even more difficult to predict?
    */
@@ -2368,6 +2894,7 @@ void mutt_init (int skip_sys_rc, LIST *commands)
   mutt_set_langinfo_charset ();
   mutt_set_charset (Charset);
   
+  Matches = safe_calloc (Matches_listsize, sizeof (char *));
   
   /* Set standard defaults */
   for (i = 0; MuttVars[i].option; i++)
@@ -2499,3 +3026,81 @@ int mutt_get_hook_type (const char *name)
       return c->data;
   return 0;
 }
+
+static int parse_group_context (group_context_t **ctx, BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
+{
+  while (!mutt_strcasecmp (buf->data, "-group"))
+  {
+    if (!MoreArgs (s)) 
+    {
+      strfcpy (err->data, _("-group: no group name"), err->dsize);
+      goto bail;
+    }
+    
+    mutt_extract_token (buf, s, 0);
+
+    mutt_group_context_add (ctx, mutt_pattern_group (buf->data));
+    
+    if (!MoreArgs (s))
+    {
+      strfcpy (err->data, _("out of arguments"), err->dsize);
+      goto bail;
+    }
+    
+    mutt_extract_token (buf, s, 0);
+  }
+  
+  return 0;
+  
+  bail:
+  mutt_group_context_destroy (ctx);
+  return -1;
+}
+
+static void myvar_set (const char* var, const char* val)
+{
+  myvar_t** cur;
+
+  for (cur = &MyVars; *cur; cur = &((*cur)->next))
+    if (!mutt_strcmp ((*cur)->name, var))
+      break;
+
+  if (!*cur)
+    *cur = safe_calloc (1, sizeof (myvar_t));
+  
+  if (!(*cur)->name)
+    (*cur)->name = safe_strdup (var);
+  
+  mutt_str_replace (&(*cur)->value, val);
+}
+
+static void myvar_del (const char* var)
+{
+  myvar_t **cur;
+  myvar_t *tmp;
+  
+
+  for (cur = &MyVars; *cur; cur = &((*cur)->next))
+    if (!mutt_strcmp ((*cur)->name, var))
+      break;
+  
+  if (*cur) 
+  {
+    tmp = (*cur)->next;
+    FREE (&(*cur)->name);
+    FREE (&(*cur)->value);
+    FREE (cur);		/* __FREE_CHECKED__ */
+    *cur = tmp;
+  }
+}
+
+static const char* myvar_get (const char* var)
+{
+  myvar_t* cur;
+
+  for (cur = MyVars; cur; cur = cur->next)
+    if (!mutt_strcmp (cur->name, var))
+      return NONULL(cur->value);
+
+  return NULL;
+}
