@@ -13,7 +13,7 @@
  *
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 /*
@@ -469,3 +469,36 @@ wchar_t replacement_char (void)
 {
   return Charset_is_utf8 ? 0xfffd : '?';
 }
+
+int mutt_filter_unprintable (char **s)
+{
+  BUFFER *b = NULL;
+  wchar_t wc;
+  size_t k, k2;
+  char scratch[MB_LEN_MAX + 1];
+  char *p = *s;
+  mbstate_t mbstate1, mbstate2;
+
+  if (!(b = mutt_buffer_init (b)))
+    return -1;
+  memset (&mbstate1, 0, sizeof (mbstate1));
+  memset (&mbstate2, 0, sizeof (mbstate2));
+  for (; (k = mbrtowc (&wc, p, MB_LEN_MAX, &mbstate1)); p += k)
+  {
+    if (k == (size_t)(-1) || k == (size_t)(-2))
+    {
+      k = 1;
+      wc = replacement_char();
+    }
+    if (!IsWPrint (wc))
+      wc = '?';
+    k2 = wcrtomb (scratch, wc, &mbstate2);
+    scratch[k2] = '\0';
+    mutt_buffer_addstr (b, scratch);
+  }
+  FREE (s);  /* __FREE_CHECKED__ */
+  *s = b->data ? b->data : safe_calloc (1, 1);
+  FREE (&b);
+  return 0;
+}
+
