@@ -13,7 +13,7 @@
  *
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #if HAVE_CONFIG_H
@@ -379,24 +379,28 @@ void menu_check_recenter (MUTTMENU *menu)
 
   if (!option (OPTMENUMOVEOFF) && menu->max <= menu->pagelen) /* less entries than lines */
   {
-    if (menu->top != 0) {
+    if (menu->top != 0) 
+    {
       menu->top = 0;
       set_option (OPTNEEDREDRAW);
     }
   }
-  else if (menu->current >= menu->top + menu->pagelen - c) /* indicator below bottom threshold */
+  else 
   {
-    if (option (OPTMENUSCROLL) || (menu->pagelen <= 0))
-      menu->top = menu->current - menu->pagelen + c + 1;
+    if (option (OPTMENUSCROLL) || (menu->pagelen <= 0) || (c < MenuContext))
+    {
+      if (menu->current < menu->top + c)
+	menu->top = menu->current - c;
+      else if (menu->current >= menu->top + menu->pagelen - c)
+	menu->top = menu->current - menu->pagelen + c + 1;
+    }
     else
-      menu->top += (menu->pagelen - c) * ((menu->current - menu->top) / (menu->pagelen - c)) - c;
-  }
-  else if (menu->current < menu->top + c) /* indicator above top threshold */
-  {
-    if (option (OPTMENUSCROLL) || (menu->pagelen <= 0))
-      menu->top = menu->current - c;
-    else
-      menu->top -= (menu->pagelen - c) * ((menu->top + menu->pagelen - 1 - menu->current) / (menu->pagelen - c)) - c;
+    {
+      if (menu->current < menu->top + c)
+	menu->top -= (menu->pagelen - c) * ((menu->top + menu->pagelen - 1 - menu->current) / (menu->pagelen - c)) - c;
+      else if ((menu->current >= menu->top + menu->pagelen - c))
+	menu->top += (menu->pagelen - c) * ((menu->current - menu->top) / (menu->pagelen - c)) - c;	
+    }
   }
 
   if (!option (OPTMENUMOVEOFF)) /* make entries stick to bottom */
@@ -699,7 +703,7 @@ void mutt_menuDestroy (MUTTMENU **p)
     FREE (& (*p)->dialog);
   }
 
-  FREE (p);
+  FREE (p);		/* __FREE_CHECKED__ */
 }
 
 #define M_SEARCH_UP   1
