@@ -208,6 +208,7 @@ enum
   /* actions for mutt_pattern_comp/mutt_pattern_exec */
   M_AND,
   M_OR,
+  M_THREAD,
   M_TO,
   M_CC,
   M_COLLAPSED,
@@ -352,9 +353,9 @@ enum
   OPTFORCENAME,
   OPTFORWDECODE,
   OPTFORWQUOTE,
-#if USE_HCACHE
+#ifdef USE_HCACHE
   OPTHCACHEVERIFY,
-#if HAVE_QDBM
+#ifdef HAVE_QDBM
   OPTHCACHECOMPRESS,
 #endif /* HAVE_QDBM */
 #endif
