@@ -14,7 +14,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */ 
 
 #ifndef _MUTT_CURSES_H_
@@ -26,7 +26,9 @@
 #define unix
 #endif /* unix */
 
-#include "slcurses.h"
+#include <slang.h>	/* in addition to slcurses.h, we need slang.h for the version
+			   number to test for 2.x having UTF-8 support in main.c */
+#include <slcurses.h>
 
 #define KEY_DC SL_KEY_DELETE
 #define KEY_IC SL_KEY_IC
@@ -38,16 +40,16 @@
 #define M_ENTER_C '\r'
 #define M_ENTER_S "\r"
 
-#else
+#else /* USE_SLANG_CURSES */
 
-#ifdef HAVE_NCURSESW_NCURSES_H
-#include <ncursesw/ncurses.h>
+#if HAVE_NCURSESW_NCURSES_H
+# include <ncursesw/ncurses.h>
+#elif HAVE_NCURSES_NCURSES_H
+# include <ncurses/ncurses.h>
+#elif HAVE_NCURSES_H
+# include <ncurses.h>
 #else
-#ifdef HAVE_NCURSES_H
-#include <ncurses.h>
-#else
-#include <curses.h>
-#endif
+# include <curses.h>
 #endif
 
 #define M_ENTER_C '\n'
