@@ -13,7 +13,7 @@
  *
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #if HAVE_CONFIG_H
@@ -72,7 +72,7 @@ static int tls_init (void)
   return 0;
 }
 
-int mutt_gnutls_socket_setup (CONNECTION* conn)
+int mutt_ssl_socket_setup (CONNECTION* conn)
 {
   if (tls_init() < 0)
     return -1;
@@ -81,6 +81,7 @@ int mutt_gnutls_socket_setup (CONNECTION* conn)
   conn->conn_read	= tls_socket_read;
   conn->conn_write	= tls_socket_write;
   conn->conn_close	= tls_socket_close;
+  conn->conn_poll       = raw_socket_poll;
 
   return 0;
 }
@@ -143,7 +144,7 @@ static int tls_socket_open (CONNECTION* conn)
   return 0;
 }
 
-int mutt_gnutls_starttls (CONNECTION* conn)
+int mutt_ssl_starttls (CONNECTION* conn)
 {
   if (tls_init() < 0)
     return -1;
@@ -287,7 +288,7 @@ static int tls_socket_close (CONNECTION* conn)
 
     gnutls_certificate_free_credentials (data->xcred);
     gnutls_deinit (data->state);
-    safe_free ((void **) &conn->sockdata);
+    FREE (&conn->sockdata);
   }
 
   return raw_socket_close (conn);
@@ -440,7 +441,7 @@ static int tls_check_stored_hostname (const gnutls_datum *cert,
               strcmp(linestr + pmatch[2].rm_so, buf) == 0)
           {
             regfree(&preg);
-            safe_free((void**)&linestr);
+            FREE(&linestr);
             fclose(fp);
             return 1;
           }
