@@ -14,7 +14,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */ 
 
 #if HAVE_CONFIG_H
@@ -28,8 +28,6 @@
 #include "mx.h"
 #include "url.h"
 
-#include "reldate.h"
-
 #ifdef USE_IMAP
 #include "imap.h"
 #endif
@@ -291,7 +289,7 @@ void mutt_free_header (HEADER **h)
 #if defined USE_POP || defined USE_IMAP
   FREE (&(*h)->data);
 #endif
-  FREE (h);
+  FREE (h);		/* __FREE_CHECKED__ */
 }
 
 /* returns true if the header contained in "s" is in list "t" */
@@ -679,7 +677,7 @@ void mutt_free_envelope (ENVELOPE **p)
   mutt_free_list (&(*p)->references);
   mutt_free_list (&(*p)->in_reply_to);
   mutt_free_list (&(*p)->userhdrs);
-  FREE (p);
+  FREE (p);		/* __FREE_CHECKED__ */
 }
 
 /* move all the headers from extra not present in base into base */
@@ -913,13 +911,13 @@ int mutt_check_overwrite (const char *attname, const char *path,
 	  mutt_str_replace (directory, fname);
 	  break;
 	case 1:		/* yes */
-	  FREE (directory);
+	  FREE (directory);		/* __FREE_CHECKED__ */
 	  break;
 	case -1:	/* abort */
-	  FREE (directory); 
+	  FREE (directory); 		/* __FREE_CHECKED__ */
 	  return -1;
 	case  2:	/* no */
-	  FREE (directory);
+	  FREE (directory);		/* __FREE_CHECKED__ */
 	  return 1;
       }
     }
@@ -1393,7 +1391,7 @@ BUFFER * mutt_buffer_init(BUFFER *b)
   }
   else
   {
-    safe_free(b->data);
+    FREE(&b->data);
   }
   memset(b, 0, sizeof(BUFFER));
   return b;
@@ -1419,6 +1417,43 @@ BUFFER * mutt_buffer_from(BUFFER *b, char *seed)
   return b;
 }
 
+int mutt_buffer_printf (BUFFER* buf, const char* fmt, ...)
+{
+  va_list ap, ap_retry;
+  int len, blen, doff;
+  
+  va_start (ap, fmt);
+  va_copy (ap_retry, ap);
+
+  doff = buf->dptr - buf->data;
+  blen = buf->dsize - doff;
+  /* solaris 9 vsnprintf barfs when blen is 0 */
+  if (!blen)
+  {
+    blen = 128;
+    buf->dsize += blen;
+    safe_realloc (&buf->data, buf->dsize);
+    buf->dptr = buf->data + doff;
+  }
+  if ((len = vsnprintf (buf->dptr, blen, fmt, ap)) >= blen)
+  {
+    blen = ++len - blen;
+    if (blen < 128)
+      blen = 128;
+    buf->dsize += blen;
+    safe_realloc (&buf->data, buf->dsize);
+    buf->dptr = buf->data + doff;
+    len = vsnprintf (buf->dptr, len, fmt, ap_retry);
+    va_end (ap_retry);
+  }
+  if (len > 0)
+    buf->dptr += len;
+
+  va_end (ap);
+  
+  return len;
+}
+
 void mutt_buffer_addstr (BUFFER* buf, const char* s)
 {
   mutt_buffer_add (buf, s, mutt_strlen (s));
@@ -1436,7 +1471,7 @@ void mutt_buffer_free (BUFFER **p)
 
    FREE(&(*p)->data);
    /* dptr is just an offset to data and shouldn't be freed */
-   FREE(p);
+   FREE(p);		/* __FREE_CHECKED__ */
 }
 
 /* dynamically grows a BUFFER to accomodate s, in increments of 128 bytes.
@@ -1508,7 +1543,7 @@ void mutt_free_regexp (REGEXP **pp)
   FREE (&(*pp)->pattern);
   regfree ((*pp)->rx);
   FREE (&(*pp)->rx);
-  FREE (pp);
+  FREE (pp);		/* __FREE_CHECKED__ */
 }
 
 void mutt_free_rx_list (RX_LIST **list)
@@ -1535,7 +1570,7 @@ void mutt_free_spam_list (SPAM_LIST **list)
     p = *list;
     *list = (*list)->next;
     mutt_free_regexp (&p->rx);
-    safe_free(&p->template);
+    FREE (&p->template);
     FREE (&p);
   }
 }
@@ -1588,7 +1623,7 @@ int mutt_match_spam_list (const char *s, SPAM_LIST *l, char *text, int x)
 	if (*p == '%')
 	{
 	  n = atoi(++p);			/* find pmatch index */
-	  while (isdigit(*p))
+	  while (isdigit((unsigned char)*p))
 	    ++p;				/* skip subst token */
 	  for (i = pmatch[n].rm_so; (i < pmatch[n].rm_eo) && (tlen < x); i++)
 	    text[tlen++] = s[i];
