@@ -15,7 +15,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */ 
 
 /*
@@ -106,7 +106,15 @@ void pgp_forget_passphrase (void)
 
 int pgp_use_gpg_agent (void)
 {
-  return option (OPTUSEGPGAGENT) && getenv ("GPG_TTY") && getenv ("GPG_AGENT_INFO");
+  char *tty;
+
+  if (!option (OPTUSEGPGAGENT) || !getenv ("GPG_AGENT_INFO"))
+    return 0;
+
+  if ((tty = ttyname(0)))
+    setenv("GPG_TTY", tty, 0);
+
+  return 1;
 }
 
 char *pgp_keyid(pgp_key_t k)
@@ -235,11 +243,13 @@ static void pgp_copy_clearsigned (FILE *fpin, STATE *s, char *charset)
 
 int pgp_application_pgp_handler (BODY *m, STATE *s)
 {
+  int could_not_decrypt = 0;
   int needpass = -1, pgp_keyblock = 0;
   int clearsign = 0, rv, rc;
   int c = 1; /* silence GCC warning */
   long start_pos = 0;
-  long bytes, last_pos, offset;
+  long bytes;
+  LOFF_T last_pos, offset;
   char buf[HUGE_STRING];
   char outfile[_POSIX_PATH_MAX];
   char tmpfname[_POSIX_PATH_MAX];
@@ -255,7 +265,7 @@ int pgp_application_pgp_handler (BODY *m, STATE *s)
 
   rc = 0;	/* silence false compiler warning if (s->flags & M_DISPLAY) */
 
-  fseek (s->fpin, m->offset, 0);
+  fseeko (s->fpin, m->offset, 0);
   last_pos = m->offset;
   
   for (bytes = m->length; bytes > 0;)
@@ -263,7 +273,7 @@ int pgp_application_pgp_handler (BODY *m, STATE *s)
     if (fgets (buf, sizeof (buf), s->fpin) == NULL)
       break;
     
-    offset = ftell (s->fpin);
+    offset = ftello (s->fpin);
     bytes -= (offset - last_pos); /* don't rely on mutt_strlen(buf) */
     last_pos = offset;
     
@@ -307,7 +317,7 @@ int pgp_application_pgp_handler (BODY *m, STATE *s)
       fputs (buf, tmpfp);
       while (bytes > 0 && fgets (buf, sizeof (buf) - 1, s->fpin) != NULL)
       {
-	offset = ftell (s->fpin);
+	offset = ftello (s->fpin);
 	bytes -= (offset - last_pos); /* don't rely on mutt_strlen(buf) */
 	last_pos = offset;
 	
@@ -322,8 +332,7 @@ int pgp_application_pgp_handler (BODY *m, STATE *s)
 
       /* leave tmpfp open in case we still need it - but flush it! */
       fflush (tmpfp);
-      
-      
+
       /* Invoke PGP if needed */
       if (!clearsign || (s->flags & M_VERIFY))
       {
@@ -381,6 +390,7 @@ int pgp_application_pgp_handler (BODY *m, STATE *s)
 	}
 	
         /* treat empty result as sign of failure */
+	/* TODO: maybe on failure mutt should include the original undecoded text. */
 	if (pgpout)
 	{
 	  rewind (pgpout);
@@ -388,13 +398,17 @@ int pgp_application_pgp_handler (BODY *m, STATE *s)
 	  ungetc (c, pgpout);
 	}
         if (!clearsign && (!pgpout || c == EOF))
+	{
+	  could_not_decrypt = 1;
+	  pgp_void_passphrase ();
+	}
+	
+	if (could_not_decrypt && !(s->flags & M_DISPLAY))
 	{
           mutt_error _("Could not decrypt PGP message");
 	  mutt_sleep (1);
-          pgp_void_passphrase ();
-          rc = -1;
-          
-          goto out;
+	  rc = -1;
+	  goto out;
         }
       }
       
@@ -438,7 +452,10 @@ int pgp_application_pgp_handler (BODY *m, STATE *s)
 	if (needpass)
         {
 	  state_attach_puts (_("[-- END PGP MESSAGE --]\n"), s);
-          mutt_message _("PGP message successfully decrypted.");
+	  if (could_not_decrypt)
+	    mutt_error _("Could not decrypt PGP message");
+	  else
+	    mutt_message _("PGP message successfully decrypted.");
         }
 	else if (pgp_keyblock)
 	  state_attach_puts (_("[-- END PGP PUBLIC KEY BLOCK --]\n"), s);
@@ -580,7 +597,7 @@ int pgp_verify_one (BODY *sigbdy, STATE *s, const char *tempfile)
     return -1;
   }
 	
-  fseek (s->fpin, sigbdy->offset, 0);
+  fseeko (s->fpin, sigbdy->offset, 0);
   mutt_copy_bytes (s->fpin, fp, sigbdy->length);
   fclose (fp);
   
@@ -776,7 +793,7 @@ BODY *pgp_decrypt_part (BODY *a, STATE *s, FILE *fpout, BODY *p)
    * the temporary file.
    */
 
-  fseek (s->fpin, a->offset, 0);
+  fseeko (s->fpin, a->offset, 0);
   mutt_copy_bytes (s->fpin, pgptmp, a->length);
   fclose (pgptmp);
 
