@@ -13,8 +13,8 @@
  * 
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
- * MA 02111, USA.
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ * MA  02110-1301, USA.
  */
 
 /* This module peeks at a PGP signature and figures out the hash
@@ -68,8 +68,8 @@ static const char *pgp_hash_to_micalg (short id)
 static void pgp_dearmor (FILE *in, FILE *out)
 {
   char line[HUGE_STRING];
-  long start;
-  long end;
+  LOFF_T start;
+  LOFF_T end;
   char *r;
 
   STATE state;
@@ -105,7 +105,7 @@ static void pgp_dearmor (FILE *in, FILE *out)
   }
   
   /* actual data starts here */
-  start = ftell (in);
+  start = ftello (in);
   
   /* find the checksum */
   
@@ -120,13 +120,13 @@ static void pgp_dearmor (FILE *in, FILE *out)
     return;
   }
   
-  if ((end = ftell (in) - strlen (line)) < start)
+  if ((end = ftello (in) - strlen (line)) < start)
   {
     dprint (1, (debugfile, "pgp_dearmor: end < start???\n"));
     return;
   }
   
-  if (fseek (in, start, SEEK_SET) == -1)
+  if (fseeko (in, start, SEEK_SET) == -1)
   {
     dprint (1, (debugfile, "pgp_dearmor: Can't seekto start.\n"));
     return;
