@@ -13,7 +13,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */ 
 
 #ifndef _POP_H
@@ -22,6 +22,7 @@
 #include "mailbox.h"
 #include "mutt_socket.h"
 #include "mutt_curses.h"
+#include "bcache.h"
 
 #define POP_PORT 110
 #define POP_SSL_PORT 995
@@ -74,6 +75,7 @@ typedef struct
   time_t login_delay;		/* minimal login delay  capability */
   char *auth_list;		/* list of auth mechanisms */
   char *timestamp;
+  body_cache_t *bcache;		/* body cache */
   char err_msg[POP_CMD_RESPONSE];
   POP_CACHE cache[POP_CACHE_LEN];
 } POP_DATA;
