@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,9 +13,13 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */ 
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include <string.h>
 #include <ctype.h>
 #include <stdlib.h>
@@ -33,6 +37,14 @@
 #include "rfc822.h"
 #endif
 
+#include "mutt_idna.h"
+
+#define terminate_string(a, b, c) do { if ((b) < (c)) a[(b)] = 0; else \
+	a[(c)] = 0; } while (0)
+
+#define terminate_buffer(a, b) terminate_string(a, b, sizeof (a) - 1)
+
+
 const char RFC822Specials[] = "@.,:;<>[]\\\"()";
 #define is_special(x) strchr(RFC822Specials,x)
 
@@ -130,14 +142,21 @@ parse_quote (const char *s, char *token, size_t *tokenlen, size_t tokenmax)
   while (*s)
   {
     if (*tokenlen < tokenmax)
-      token[(*tokenlen)++] = *s;
+      token[*tokenlen] = *s;
     if (*s == '"')
+    {
+      (*tokenlen)++;
       return (s + 1);
+    }
     if (*s == '\\')
     {
       if (!*++s)
 	break;
+
+      if (*tokenlen < tokenmax)
+	token[*tokenlen] = *s;
     }
+    (*tokenlen)++;
     s++;
   }
   RFC822Error = ERR_MISMATCH_QUOTE;
@@ -159,7 +178,7 @@ next_token (const char *s, char *token, size_t *tokenlen, size_t tokenmax)
   }
   while (*s)
   {
-    if (ISSPACE (*s) || is_special (*s))
+    if (ISSPACE ((unsigned char) *s) || is_special (*s))
       break;
     if (*tokenlen < tokenmax)
       token[(*tokenlen)++] = *s;
@@ -220,12 +239,12 @@ parse_address (const char *s,
       return NULL;
   }
 
-  token[*tokenlen] = 0;
+  terminate_string (token, *tokenlen, tokenmax);
   addr->mailbox = safe_strdup (token);
 
   if (*commentlen && !addr->personal)
   {
-    comment[*commentlen] = 0;
+    terminate_string (comment, *commentlen, commentmax);
     addr->personal = safe_strdup (comment);
   }
 
@@ -249,7 +268,7 @@ parse_route_addr (const char *s,
     {
       if (tokenlen < sizeof (token) - 1)
 	token[tokenlen++] = '@';
-      s = parse_mailboxdomain (s + 1, ".\\[](", token,
+      s = parse_mailboxdomain (s + 1, ",.\\[](", token,
 			       &tokenlen, sizeof (token) - 1,
 			       comment, commentlen, commentmax);
     }
@@ -267,12 +286,15 @@ parse_route_addr (const char *s,
   if ((s = parse_address (s, token, &tokenlen, sizeof (token) - 1, comment, commentlen, commentmax, addr)) == NULL)
     return NULL;
 
-  if (*s != '>' || !addr->mailbox)
+  if (*s != '>')
   {
     RFC822Error = ERR_BAD_ROUTE_ADDR;
     return NULL;
   }
 
+  if (!addr->mailbox)
+    addr->mailbox = safe_strdup ("@");
+
   s++;
   return s;
 }
@@ -301,7 +323,10 @@ add_addrspec (ADDRESS **top, ADDRESS **last, const char *phrase,
   ADDRESS *cur = rfc822_new_address ();
   
   if (parse_addr_spec (phrase, comment, commentlen, commentmax, cur) == NULL)
+  {
+    rfc822_free_address (&cur);
     return;
+  }
 
   if (*last)
     (*last)->next = cur;
@@ -312,6 +337,7 @@ add_addrspec (ADDRESS **top, ADDRESS **last, const char *phrase,
 
 ADDRESS *rfc822_parse_adrlist (ADDRESS *top, const char *s)
 {
+  int ws_pending;
   const char *begin, *ps;
   char comment[STRING], phrase[STRING];
   size_t phraselen = 0, commentlen = 0;
@@ -323,6 +349,8 @@ ADDRESS *rfc822_parse_adrlist (ADDRESS *top, const char *s)
   while (last && last->next)
     last = last->next;
 
+  ws_pending = isspace ((unsigned char) *s);
+  
   SKIPWS (s);
   begin = s;
   while (*s)
@@ -331,12 +359,12 @@ ADDRESS *rfc822_parse_adrlist (ADDRESS *top, const char *s)
     {
       if (phraselen)
       {
-	phrase[phraselen] = 0;
+	terminate_buffer (phrase, phraselen);
 	add_addrspec (&top, &last, phrase, comment, &commentlen, sizeof (comment) - 1);
       }
       else if (commentlen && last && !last->personal)
       {
-	comment[commentlen] = 0;
+	terminate_buffer (comment, commentlen);
 	last->personal = safe_strdup (comment);
       }
 
@@ -364,7 +392,7 @@ ADDRESS *rfc822_parse_adrlist (ADDRESS *top, const char *s)
     else if (*s == ':')
     {
       cur = rfc822_new_address ();
-      phrase[phraselen] = 0;
+      terminate_buffer (phrase, phraselen);
       cur->mailbox = safe_strdup (phrase);
       cur->group = 1;
 
@@ -388,12 +416,12 @@ ADDRESS *rfc822_parse_adrlist (ADDRESS *top, const char *s)
     {
       if (phraselen)
       {
-	phrase[phraselen] = 0;
+	terminate_buffer (phrase, phraselen);
 	add_addrspec (&top, &last, phrase, comment, &commentlen, sizeof (comment) - 1);
       }
       else if (commentlen && last && !last->personal)
       {
-	comment[commentlen] = 0;
+	terminate_buffer (comment, commentlen);
 	last->personal = safe_strdup (comment);
       }
 #ifdef EXACT_ADDRESS
@@ -417,7 +445,7 @@ ADDRESS *rfc822_parse_adrlist (ADDRESS *top, const char *s)
     }
     else if (*s == '<')
     {
-      phrase[phraselen] = 0;
+      terminate_buffer (phrase, phraselen);
       cur = rfc822_new_address ();
       if (phraselen)
       {
@@ -446,7 +474,7 @@ ADDRESS *rfc822_parse_adrlist (ADDRESS *top, const char *s)
     }
     else
     {
-      if (phraselen && phraselen < sizeof (phrase) - 1)
+      if (phraselen && phraselen < sizeof (phrase) - 1 && ws_pending)
 	phrase[phraselen++] = ' ';
       if ((ps = next_token (s, phrase, &phraselen, sizeof (phrase) - 1)) == NULL)
       {
@@ -455,18 +483,19 @@ ADDRESS *rfc822_parse_adrlist (ADDRESS *top, const char *s)
       }
       s = ps;
     }
+    ws_pending = isspace ((unsigned char) *s);
     SKIPWS (s);
   }
   
   if (phraselen)
   {
-    phrase[phraselen] = 0;
-    comment[commentlen] = 0;
+    terminate_buffer (phrase, phraselen);
+    terminate_buffer (comment, commentlen);
     add_addrspec (&top, &last, phrase, comment, &commentlen, sizeof (comment) - 1);
   }
   else if (commentlen && last && !last->personal)
   {
-    comment[commentlen] = 0;
+    terminate_buffer (comment, commentlen);
     last->personal = safe_strdup (comment);
   }
 #ifdef EXACT_ADDRESS
@@ -485,8 +514,8 @@ void rfc822_qualify (ADDRESS *addr, const char *host)
     if (!addr->group && addr->mailbox && strchr (addr->mailbox, '@') == NULL)
     {
       p = safe_malloc (mutt_strlen (addr->mailbox) + mutt_strlen (host) + 2);
-      sprintf (p, "%s@%s", addr->mailbox, host);
-      safe_free ((void **) &addr->mailbox);
+      sprintf (p, "%s@%s", addr->mailbox, host);	/* __SPRINTF_CHECKED__ */
+      FREE (&addr->mailbox);
       addr->mailbox = p;
     }
 }
@@ -518,7 +547,8 @@ rfc822_cat (char *buf, size_t buflen, const char *value, const char *specials)
     strfcpy (buf, value, buflen);
 }
 
-void rfc822_write_address_single (char *buf, size_t buflen, ADDRESS *addr)
+void rfc822_write_address_single (char *buf, size_t buflen, ADDRESS *addr,
+				  int display)
 {
   size_t len;
   char *pbuf = buf;
@@ -605,8 +635,21 @@ void rfc822_write_address_single (char *buf, size_t buflen, ADDRESS *addr)
   {
     if (!buflen)
       goto done;
-    strfcpy (pbuf, addr->mailbox, buflen);
-    len = mutt_strlen (pbuf);
+    if (ascii_strcmp (addr->mailbox, "@") && !display)
+    {
+      strfcpy (pbuf, addr->mailbox, buflen);
+      len = mutt_strlen (pbuf);
+    }
+    else if (ascii_strcmp (addr->mailbox, "@") && display)
+    {
+      strfcpy (pbuf, mutt_addr_for_display (addr), buflen);
+      len = mutt_strlen (pbuf);
+    }
+    else
+    {
+      *pbuf = '\0';
+      len = 0;
+    }
     pbuf += len;
     buflen -= len;
 
@@ -644,7 +687,7 @@ done:
 }
 
 /* note: it is assumed that `buf' is nul terminated! */
-void rfc822_write_address (char *buf, size_t buflen, ADDRESS *addr)
+void rfc822_write_address (char *buf, size_t buflen, ADDRESS *addr, int display)
 {
   char *pbuf = buf;
   size_t len = mutt_strlen (buf);
@@ -672,7 +715,7 @@ void rfc822_write_address (char *buf, size_t buflen, ADDRESS *addr)
   {
     /* use buflen+1 here because we already saved space for the trailing
        nul char, and the subroutine can make use of it */
-    rfc822_write_address_single (pbuf, buflen + 1, addr);
+    rfc822_write_address_single (pbuf, buflen + 1, addr, display);
 
     /* this should be safe since we always have at least 1 char passed into
        the above call, which means `pbuf' should always be nul terminated */
@@ -749,9 +792,9 @@ ADDRESS *rfc822_append (ADDRESS **a, ADDRESS *b)
 }
 
 #ifdef TESTING
-int safe_free (void **p)
+int safe_free (void **p)	/* __SAFE_FREE_CHECKED__ */
 {
-  free(*p);
+  free(*p);		/* __MEM_CHECKED__ */
   *p = 0;
 }
 
@@ -760,7 +803,7 @@ int main (int argc, char **argv)
   ADDRESS *list;
   char buf[256];
 # if 0
-  char *str = "michael, Michael Elkins <me@cs.hmc.edu>, testing a really complex address: this example <@contains.a.source.route@with.multiple.hosts:address@example.com>;, lothar@of.the.hillpeople (lothar)";
+  char *str = "michael, Michael Elkins <me@mutt.org>, testing a really complex address: this example <@contains.a.source.route,@with.multiple.hosts:address@example.com>;, lothar@of.the.hillpeople (lothar)";
 # else
   char *str = "a b c ";
 # endif
