@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,13 +13,15 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */ 
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "mutt.h"
-#include "mutt_regex.h"
 #include "sort.h"
-#include "pattern.h"
 #include <string.h>
 #include <stdlib.h>
 
@@ -34,69 +36,92 @@ typedef struct score_t
 
 SCORE *Score = NULL;
 
-static void score_CheckResort (void)
+void mutt_check_rescore (CONTEXT *ctx)
 {
-  /* Conditions under which we should resort the mailbox */
-  if ((Sort & SORT_MASK) == SORT_SCORE || (SortAux & SORT_MASK) == SORT_SCORE)
+  int i;
+
+  if (option (OPTNEEDRESCORE) && option (OPTSCORE))
   {
-    set_option (OPTNEEDRESORT);
-    set_option (OPTNEEDRESCORE);
-    if ((Sort & SORT_MASK) == SORT_THREADS)
-      set_option (OPTSORTSUBTHREADS);
+    if ((Sort & SORT_MASK) == SORT_SCORE ||
+	(SortAux & SORT_MASK) == SORT_SCORE)
+    {
+      set_option (OPTNEEDRESORT);
+      if ((Sort & SORT_MASK) == SORT_THREADS)
+	set_option (OPTSORTSUBTHREADS);
+    }
+
+    /* must redraw the index since the user might have %N in it */
+    set_option (OPTFORCEREDRAWINDEX);
+    set_option (OPTFORCEREDRAWPAGER);
+
+    for (i = 0; ctx && i < ctx->msgcount; i++)
+    {
+      mutt_score_message (ctx, ctx->hdrs[i], 1);
+      ctx->hdrs[i]->pair = 0;
+    }
   }
+  unset_option (OPTNEEDRESCORE);
 }
 
-int mutt_parse_score (const char *s, unsigned long data, char *errmsg, size_t errlen)
+int mutt_parse_score (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
 {
-  char pattern[LONG_STRING];
-  char score[SHORT_STRING];
-  char expn[LONG_STRING];
-  char *pc;
   SCORE *ptr, *last;
+  char *pattern, *pc;
   struct pattern_t *pat;
 
-  s = mutt_extract_token (pattern, sizeof (pattern), s, expn, sizeof (expn), 0);
-  if (!s)
+  mutt_extract_token (buf, s, 0);
+  if (!MoreArgs (s))
   {
-    strfcpy (errmsg, "too few arguments", errlen);
+    strfcpy (err->data, _("score: too few arguments"), err->dsize);
     return (-1);
   }
-  s = mutt_extract_token (score, sizeof (score), s, expn, sizeof (expn), 0);
-  if (s)
+  pattern = buf->data;
+  memset (buf, 0, sizeof (BUFFER));
+  mutt_extract_token (buf, s, 0);
+  if (MoreArgs (s))
   {
-    strfcpy (errmsg, "too many arguments", errlen);
+    FREE (&pattern);
+    strfcpy (err->data, _("score: too many arguments"), err->dsize);
     return (-1);
   }
 
   /* look for an existing entry and update the value, else add it to the end
      of the list */
   for (ptr = Score, last = NULL; ptr; last = ptr, ptr = ptr->next)
-    if (strcmp (pattern, ptr->str) == 0)
+    if (mutt_strcmp (pattern, ptr->str) == 0)
       break;
   if (!ptr)
   {
-    if ((pat = mutt_pattern_comp (pattern, 0, errmsg, errlen)) == NULL)
+    if ((pat = mutt_pattern_comp (pattern, 0, err)) == NULL)
+    {
+      FREE (&pattern);
       return (-1);
+    }
     ptr = safe_calloc (1, sizeof (SCORE));
     if (last)
       last->next = ptr;
     else
       Score = ptr;
     ptr->pat = pat;
-    ptr->str = safe_strdup (pattern);
-  }
-  pc = score;
+    ptr->str = pattern;
+  } else
+    /* 'buf' arg was cleared and 'pattern' holds the only reference;
+     * as here 'ptr' != NULL -> update the value only in which case
+     * ptr->str already has the string, so pattern should be freed.
+     */
+    FREE (&pattern);
+  pc = buf->data;
   if (*pc == '=')
   {
     ptr->exact = 1;
     pc++;
   }
   ptr->val = atoi (pc);
-  score_CheckResort ();
+  set_option (OPTNEEDRESCORE);
   return 0;
 }
 
-void mutt_score_message (HEADER *hdr)
+void mutt_score_message (CONTEXT *ctx, HEADER *hdr, int upd_ctx)
 {
   SCORE *tmp;
 
@@ -115,47 +140,51 @@ void mutt_score_message (HEADER *hdr)
   }
   if (hdr->score < 0)
     hdr->score = 0;
+  
+  if (hdr->score <= ScoreThresholdDelete)
+    _mutt_set_flag (ctx, hdr, M_DELETE, 1, upd_ctx);
+  if (hdr->score <= ScoreThresholdRead)
+    _mutt_set_flag (ctx, hdr, M_READ, 1, upd_ctx);
+  if (hdr->score >= ScoreThresholdFlag)
+    _mutt_set_flag (ctx, hdr, M_FLAG, 1, upd_ctx);
 }
 
-int mutt_parse_unscore (const char *s, unsigned long data, char *errmsg, size_t errlen)
+int mutt_parse_unscore (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
 {
-  char buf[SHORT_STRING];
-  char expn[LONG_STRING];
   SCORE *tmp, *last = NULL;
 
-  while (s)
+  while (MoreArgs (s))
   {
-    s = mutt_extract_token (buf, sizeof (buf), s, expn, sizeof (expn), 0);
-    if (!strcmp ("*", buf))
+    mutt_extract_token (buf, s, 0);
+    if (!mutt_strcmp ("*", buf->data))
     {
       for (tmp = Score; tmp; )
       {
 	last = tmp;
 	tmp = tmp->next;
 	mutt_pattern_free (&last->pat);
-	safe_free ((void **) &last);
+	FREE (&last);
       }
       Score = NULL;
-      break; /* nothing else to do */
     }
     else
     {
       for (tmp = Score; tmp; last = tmp, tmp = tmp->next)
       {
-	if (!strcmp (buf, tmp->str))
+	if (!mutt_strcmp (buf->data, tmp->str))
 	{
 	  if (last)
 	    last->next = tmp->next;
 	  else
 	    Score = tmp->next;
 	  mutt_pattern_free (&tmp->pat);
-	  safe_free ((void **) &tmp);
+	  FREE (&tmp);
 	  /* there should only be one score per pattern, so we can stop here */
 	  break;
 	}
       }
     }
   }
-  score_CheckResort ();
+  set_option (OPTNEEDRESCORE);
   return 0;
 }
