@@ -13,7 +13,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */ 
 
 #if HAVE_CONFIG_H
@@ -87,7 +87,7 @@ static int mutt_addrsrc (ADDRESS *a, ADDRESS *lst)
 }
 
 /* removes addresses from "b" which are contained in "a" */
-static ADDRESS *mutt_remove_xrefs (ADDRESS *a, ADDRESS *b)
+ADDRESS *mutt_remove_xrefs (ADDRESS *a, ADDRESS *b)
 {
   ADDRESS *top, *p, *prev = NULL;
 
@@ -589,6 +589,12 @@ void mutt_fix_reply_recipients (ENVELOPE *env)
   env->to = mutt_remove_duplicates (env->to);
   env->cc = mutt_remove_duplicates (env->cc);
   env->cc = mutt_remove_xrefs (env->to, env->cc);
+  
+  if (env->cc && !env->to)
+  {
+    env->to = env->cc;
+    env->cc = NULL;
+  }
 }
 
 void mutt_make_forward_subject (ENVELOPE *env, CONTEXT *ctx, HEADER *cur)
@@ -1027,6 +1033,19 @@ static void decode_descriptions (BODY *b)
   }
 }
 
+static void fix_end_of_file (const char *data)
+{
+  FILE *fp;
+  int c;
+  
+  if ((fp = safe_fopen (data, "a+")) == NULL)
+    return;
+  fseek (fp,-1,SEEK_END);
+  if ((c = fgetc(fp)) != '\n')
+    fputc ('\n', fp);
+  safe_fclose (&fp);
+}
+
 int mutt_resend_message (FILE *fp, CONTEXT *ctx, HEADER *cur)
 {
   HEADER *msg = mutt_new_header ();
@@ -1122,8 +1141,9 @@ ci_send_message (int flags,		/* send mode */
     pbody = mutt_new_body ();
     pbody->next = msg->content; /* don't kill command-line attachments */
     msg->content = pbody;
-    
-    ctype = safe_strdup (ContentType);
+
+    if (!(ctype = safe_strdup (ContentType)))
+      ctype = safe_strdup ("text/plain");
     mutt_parse_content_type (ctype, msg->content);
     FREE (&ctype);
     msg->content->unlink = 1;
@@ -1386,7 +1406,16 @@ ci_send_message (int flags,		/* send mode */
 	mutt_env_to_idna (msg->env, NULL, NULL);
       }
       else
+      {
 	mutt_edit_file (Editor, msg->content->filename);
+	if (stat (msg->content->filename, &st) == 0)
+	{
+	  if (mtime != st.st_mtime)
+	    fix_end_of_file (msg->content->filename);
+	}
+	else
+	  mutt_perror (msg->content->filename);
+      }
       
       mutt_message_hook (NULL, msg, M_SEND2HOOK);
     }
