@@ -15,7 +15,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #if HAVE_CONFIG_H
@@ -1577,7 +1577,7 @@ int smime_verify_one (BODY *sigbdy, STATE *s, const char *tempfile)
 
   mutt_decode_attachment (sigbdy, s);
 
-  sigbdy->length = ftell (s->fpout);
+  sigbdy->length = ftello (s->fpout);
   sigbdy->offset = 0;
   fclose (s->fpout);
 
@@ -1621,7 +1621,7 @@ int smime_verify_one (BODY *sigbdy, STATE *s, const char *tempfile)
       rewind (smimeerr);
       
       line = mutt_read_line (line, &linelen, smimeerr, &lineno);
-      if (linelen && !mutt_strcasecmp (line, "verification successful"))
+      if (linelen && !ascii_strcasecmp (line, "verification successful"))
 	badsig = 0;
 
       FREE (&line);
@@ -1702,7 +1702,7 @@ static BODY *smime_handle_entity (BODY *m, STATE *s, FILE *outFile)
     return NULL;
   }
 
-  fseek (s->fpin, m->offset, 0);
+  fseeko (s->fpin, m->offset, 0);
   last_pos = m->offset;
 
   mutt_copy_bytes (s->fpin, tmpfp,  m->length);
@@ -1842,7 +1842,7 @@ static BODY *smime_handle_entity (BODY *m, STATE *s, FILE *outFile)
     rewind (smimeerr);
     
     line = mutt_read_line (line, &linelen, smimeerr, &lineno);
-    if (linelen && !mutt_strcasecmp (line, "verification successful"))
+    if (linelen && !ascii_strcasecmp (line, "verification successful"))
       m->goodsig = 1;
     FREE (&line);
   }
@@ -1880,7 +1880,7 @@ int smime_decrypt_mime (FILE *fpin, FILE **fpout, BODY *b, BODY **cur)
   
   memset (&s, 0, sizeof (s));
   s.fpin = fpin;
-  fseek (s.fpin, b->offset, 0);
+  fseeko (s.fpin, b->offset, 0);
 
   mutt_mktemp (tempfile);
   if ((tmpfp = safe_fopen (tempfile, "w+")) == NULL)
@@ -1893,7 +1893,7 @@ int smime_decrypt_mime (FILE *fpin, FILE **fpout, BODY *b, BODY **cur)
   s.fpout = tmpfp;
   mutt_decode_attachment (b, &s);
   fflush (tmpfp);
-  b->length = ftell (s.fpout);
+  b->length = ftello (s.fpout);
   b->offset = 0;
   rewind (tmpfp);
   s.fpin = tmpfp;
@@ -1950,27 +1950,71 @@ int smime_send_menu (HEADER *msg, int *redraw)
     break;
 
   case 3: /* encrypt (w)ith */
-    msg->security |= ENCRYPT;
-    switch (mutt_multi_choice (_("1: DES, 2: Triple-DES, 3: RC2-40,"
-				 " 4: RC2-64, 5: RC2-128, or (f)orget it? "),
-			       _("12345f"))) {
-    case 1:
-	mutt_str_replace (&SmimeCryptAlg, "des");
-	break;
-    case 2:
-	mutt_str_replace (&SmimeCryptAlg, "des3");
-	break;
-    case 3:
-	mutt_str_replace (&SmimeCryptAlg, "rc2-40");
-	break;
-    case 4:
-	mutt_str_replace (&SmimeCryptAlg, "rc2-64");
-	break;
-    case 5:
-	mutt_str_replace (&SmimeCryptAlg, "rc2-128");
-	break;
-    case 6: /* forget it */
-	break;
+    {
+      int choice = 0;
+
+      msg->security |= ENCRYPT;
+      do
+      {
+        /* I use "dra" because "123" is recognized anyway */
+        switch (mutt_multi_choice (_("Choose algorithm family:"
+                                     " 1: DES, 2: RC2, 3: AES,"
+                                     " or (c)lear? "),
+                                   _("drac")))
+        {
+        case 1:
+          switch (choice = mutt_multi_choice (_("1: DES, 2: Triple-DES "),
+                                              _("dt")))
+          {
+          case 1:
+            mutt_str_replace (&SmimeCryptAlg, "des");
+            break;
+          case 2:
+            mutt_str_replace (&SmimeCryptAlg, "des3");
+            break;
+          }
+          break;
+
+        case 2:
+          switch (choice = mutt_multi_choice (_("1: RC2-40, 2: RC2-64, 3: RC2-128 "),
+                                              _("468")))
+          {
+          case 1:
+            mutt_str_replace (&SmimeCryptAlg, "rc2-40");
+            break;
+          case 2:
+            mutt_str_replace (&SmimeCryptAlg, "rc2-64");
+            break;
+          case 3:
+            mutt_str_replace (&SmimeCryptAlg, "rc2-128");
+            break;
+          }
+          break;
+
+        case 3:
+          switch (choice = mutt_multi_choice (_("1: AES128, 2: AES192, 3: AES256 "),
+                                              _("895")))
+          {
+          case 1:
+            mutt_str_replace (&SmimeCryptAlg, "aes128");
+            break;
+          case 2:
+            mutt_str_replace (&SmimeCryptAlg, "aes192");
+            break;
+          case 3:
+            mutt_str_replace (&SmimeCryptAlg, "aes256");
+            break;
+          }
+          break;
+
+        case 4: /* (c)lear */
+          FREE (&SmimeCryptAlg);
+          /* fallback */
+        case -1: /* Ctrl-G or Enter */
+          choice = 0;
+          break;
+        }
+      } while (choice == -1);
     }
     break;
 
@@ -1989,7 +2033,6 @@ int smime_send_menu (HEADER *msg, int *redraw)
 
     if ((p = smime_ask_for_key (_("Sign as: "), NULL, 0))) 
     {
-      p[mutt_strlen (p)-1] = '\0';
       mutt_str_replace (&SmimeDefaultKey, p);
 	
       msg->security |= SIGN;
